# Copyright (c) 2016, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix.bindings.extensions.marking.ais as ais_binding
import stix.data_marking
from stix.data_marking import MarkingStructure


class AISConsentType(stix.Entity):
    _binding = ais_binding
    _binding_class = _binding.AISConsentType
    _namespace = 'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2'
    _ALLOWED_VALUES = ('EVERYONE', 'USG', 'NONE')

    def __init__(self, consent=None):
        self.consent = consent

    @property
    def consent(self):
        return self._consent

    @consent.setter
    def consent(self, value):
        if value is not None and value not in self._ALLOWED_VALUES:
            msg = "consent should be one of: %s. Received %s"
            raise ValueError(msg % (self._ALLOWED_VALUES, value))
        else:
            self._consent = value

    def to_obj(self, return_obj=None, ns_info=None):
        super(AISConsentType, self).to_obj(return_obj=return_obj,
                                           ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        if self.consent:
            return_obj.consent = self.consent

        return return_obj

    def to_dict(self):
        return super(AISConsentType, self).to_dict()

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.consent = obj.consent

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.consent = d.get('consent')

        return return_obj


class TLPMarkingType(stix.Entity):
    _binding = ais_binding
    _binding_class = _binding.TLPMarkingType
    _namespace = 'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2'
    _ALLOWED_VALUES = ('WHITE', 'GREEN', 'AMBER')

    def __init__(self, color=None):
        self.color = color

    @property
    def color(self):
        return self._color

    @color.setter
    def color(self, value):
        if value is not None and value not in self._ALLOWED_VALUES:
            msg = "color should be one of: %s. Received %s"
            raise ValueError(msg % (self._ALLOWED_VALUES, value))
        else:
            self._color = value

    def to_obj(self, return_obj=None, ns_info=None):
        super(TLPMarkingType, self).to_obj(return_obj=return_obj,
                                           ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        return_obj.color = self.color

        return return_obj

    def to_dict(self):
        return super(TLPMarkingType, self).to_dict()

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.color = obj.color

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.color = d.get('color')

        return return_obj


class NotProprietary(stix.Entity):
    _binding = ais_binding
    _binding_class = _binding.NotProprietary
    _namespace = 'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2'

    def __init__(self, cisa_proprietary='false', ais_consent=None,
                 tlp_marking=None):
        super(NotProprietary, self).__init__()
        self.cisa_proprietary = cisa_proprietary
        self.ais_consent = ais_consent
        self.tlp_marking = tlp_marking

    @property
    def ais_consent(self):
        return self._ais_consent

    @ais_consent.setter
    def ais_consent(self, value):
        self._set_var(AISConsentType, try_cast=False, ais_consent=value)

    @property
    def tlp_marking(self):
        return self._tlp_marking

    @tlp_marking.setter
    def tlp_marking(self, value):
        self._set_var(TLPMarkingType, try_cast=False, tlp_marking=value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(NotProprietary, self).to_obj(return_obj=return_obj,
                                           ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        if self.cisa_proprietary is not None:
            return_obj.CISA_Proprietary = self.cisa_proprietary
        if self.ais_consent:
            return_obj.AISConsent = self.ais_consent.to_obj(ns_info=ns_info)
        if self.tlp_marking:
            return_obj.TLPMarking = self.tlp_marking.to_obj(ns_info=ns_info)

        return return_obj

    def to_dict(self):
        d = {}

        if self.cisa_proprietary is not None:
            d['cisa_proprietary'] = self.cisa_proprietary
        if self.ais_consent:
            d['ais_consent'] = self.ais_consent.to_dict()
        if self.tlp_marking:
            d['tlp_marking'] = self.tlp_marking.to_dict()

        return d

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.cisa_proprietary = obj.CISA_Proprietary
        return_obj.ais_consent = AISConsentType.from_obj(obj.AISConsent)
        return_obj.tlp_marking = TLPMarkingType.from_obj(obj.TLPMarking)

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.cisa_proprietary = d.get('cisa_proprietary')
        return_obj.ais_consent = AISConsentType.from_dict(d.get('ais_consent'))
        return_obj.tlp_marking = TLPMarkingType.from_dict(d.get('tlp_marking'))

        return return_obj


class IsProprietary(stix.Entity):
    _binding = ais_binding
    _binding_class = _binding.IsProprietary
    _namespace = 'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2'

    def __init__(self, cisa_proprietary='true', ais_consent=None,
                 tlp_marking=None):
        super(IsProprietary, self).__init__()
        self.cisa_proprietary = cisa_proprietary
        self.ais_consent = ais_consent
        self.tlp_marking = tlp_marking

    @property
    def ais_consent(self):
        return self._ais_consent

    @ais_consent.setter
    def ais_consent(self, value):
        self._set_var(AISConsentType, try_cast=False, ais_consent=value)

    @property
    def tlp_marking(self):
        return self._tlp_marking

    @tlp_marking.setter
    def tlp_marking(self, value):
        self._set_var(TLPMarkingType, try_cast=False, tlp_marking=value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(IsProprietary, self).to_obj(return_obj=return_obj,
                                          ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        if self.cisa_proprietary is not None:
            return_obj.CISA_Proprietary = self.cisa_proprietary
        if self.ais_consent:
            return_obj.AISConsent = self.ais_consent.to_obj(ns_info=ns_info)
        if self.tlp_marking:
            return_obj.TLPMarking = self.tlp_marking.to_obj(ns_info=ns_info)

        return return_obj

    def to_dict(self):
        d = {}

        if self.cisa_proprietary is not None:
            d['cisa_proprietary'] = self.cisa_proprietary
        if self.ais_consent:
            d['ais_consent'] = self.ais_consent.to_dict()
        if self.tlp_marking:
            d['tlp_marking'] = self.tlp_marking.to_dict()

        return d

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.cisa_proprietary = obj.CISA_Proprietary
        return_obj.ais_consent = AISConsentType.from_obj(obj.AISConsent)
        return_obj.tlp_marking = TLPMarkingType.from_obj(obj.TLPMarking)

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.cisa_proprietary = d.get('cisa_proprietary')
        return_obj.ais_consent = AISConsentType.from_dict(d.get('ais_consent'))
        return_obj.tlp_marking = TLPMarkingType.from_dict(d.get('tlp_marking'))

        return return_obj


class AISMarkingStructure(MarkingStructure):
    _binding = ais_binding
    _binding_class = _binding.AISMarkingStructure
    _namespace = 'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2'
    _XSI_TYPE = "AIS:AISMarkingStructure"

    def __init__(self, is_proprietary=None, not_proprietary=None):
        super(AISMarkingStructure, self).__init__()
        self.is_proprietary = is_proprietary
        self.not_proprietary = not_proprietary

    @property
    def not_proprietary(self):
        return self._not_proprietary

    @not_proprietary.setter
    def not_proprietary(self, value):
        if isinstance(value, NotProprietary):
            self._not_proprietary = value
        else:
            self._set_var(NotProprietary, try_cast=False, not_proprietary=value)

    @property
    def is_proprietary(self):
        return self._is_proprietary

    @is_proprietary.setter
    def is_proprietary(self, value):
        if isinstance(value, IsProprietary):
            self._is_proprietary = value
        else:
            self._set_var(IsProprietary, try_cast=False, is_proprietary=value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(AISMarkingStructure, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        MarkingStructure.to_obj(self, return_obj=return_obj, ns_info=ns_info)
        if self.is_proprietary:
            return_obj.Is_Proprietary = self.is_proprietary.to_obj(ns_info=ns_info)
        if self.not_proprietary:
            return_obj.Not_Proprietary = self.not_proprietary.to_obj(ns_info=ns_info)

        return return_obj

    def to_dict(self):
        d = MarkingStructure.to_dict(self)

        if self.is_proprietary:
            d['is_proprietary'] = self.is_proprietary.to_dict()

        if self.not_proprietary:
            d['not_proprietary'] = self.not_proprietary.to_dict()

        return d

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        MarkingStructure.from_obj(obj, return_obj=return_obj)
        return_obj.is_proprietary = IsProprietary.from_obj(obj.Is_Proprietary)
        return_obj.not_proprietary = NotProprietary.from_obj(obj.Not_Proprietary)

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        MarkingStructure.from_dict(d, return_obj=return_obj)
        return_obj.is_proprietary = IsProprietary.from_dict(d.get('is_proprietary'))
        return_obj.not_proprietary = NotProprietary.from_dict(d.get('not_proprietary'))

        return return_obj


NAMESPACES = {
    'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2': 'AIS'
}

SCHEMALOCATIONS = {
    'http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2': 'http://www.us-cert.gov/sites/default/files/STIX_Namespace/AIS_Bundle_Marking_1.1.1_v1.0.xsd'
}


def _update_namespaces():
    # Update the python-stix namespace dictionary
    import stix.utils.nsparser as nsparser

    # Register the extension namespaces
    nsparser.DEFAULT_EXT_TO_PREFIX.update(NAMESPACES)

    # Update the default dict
    nsparser.DEFAULT_STIX_NAMESPACES.update(NAMESPACES)


def _update_schemalocations():
    # Update the python-stix schemalocation dictionary
    import stix.utils.nsparser as nsparser

    # Register the extension schemalocations
    nsparser.EXT_NS_TO_SCHEMALOCATION.update(SCHEMALOCATIONS)

    # Update the default dict
    nsparser.DEFAULT_STIX_SCHEMALOCATIONS.update(SCHEMALOCATIONS)


# Register extension
stix.data_marking.add_extension(AISMarkingStructure)
_update_namespaces()
_update_schemalocations()


# IndustryType allowed sectors
CHEMICAL_SECTOR = 'CHEMICAL SECTOR'
COMMERCIAL_FACILITIES_SECTOR = 'COMMERCIAL FACILITIES SECTOR'
COMMUNICATIONS_SECTOR = 'COMMUNICATIONS SECTOR'
CRITICAL_MANUFACTURING_SECTOR = 'CRITICAL MANUFACTURING SECTOR'
DAMS_SECTOR = 'DAMS SECTOR'
DEFENSE_INDUSTRIAL_BASE_SECTOR = 'DEFENSE INDUSTRIAL BASE SECTOR'
EMERGENCY_SERVICES_SECTOR = 'EMERGENCY SERVICES SECTOR'
ENERGY_SECTOR = 'ENERGY SECTOR'
FINANCIAL_SERVICES_SECTOR = 'FINANCIAL SERVICES SECTOR'
FOOD_AND_AGRICULTURE_SECTOR = 'FOOD AND AGRICULTURE SECTOR'
GOVERNMENT_FACILITIES_SECTOR = 'GOVERNMENT FACILITIES SECTOR'
HEALTH_CARE_AND_PUBLIC_HEALTH_SECTOR = 'HEALTHCARE AND PUBLIC HEALTH SECTOR'
INFORMATION_TECHNOLOGY_SECTOR = 'INFORMATION TECHNOLOGY SECTOR'
NUCLEAR_REACTORS_MATERIALS_AND_WASTE_SECTOR = 'NUCLEAR REACTORS MATERIALS, AND WASTE SECTOR'
TRANSPORTATION_SYSTEMS_SECTOR = 'TRANSPORTATION SYSTEMS SECTOR'
WATER_AND_WASTEWATER_SYSTEMS_SECTOR = 'WATER AND WASTEWATER SYSTEMS SECTOR'


def _validate_and_create_industry_type(industry_type):
    INDUSTRY_TYPE_SECTORS = (CHEMICAL_SECTOR, COMMERCIAL_FACILITIES_SECTOR,
                             COMMUNICATIONS_SECTOR,
                             CRITICAL_MANUFACTURING_SECTOR,
                             DAMS_SECTOR, DEFENSE_INDUSTRIAL_BASE_SECTOR,
                             EMERGENCY_SERVICES_SECTOR, ENERGY_SECTOR,
                             FINANCIAL_SERVICES_SECTOR,
                             FOOD_AND_AGRICULTURE_SECTOR,
                             GOVERNMENT_FACILITIES_SECTOR,
                             HEALTH_CARE_AND_PUBLIC_HEALTH_SECTOR,
                             INFORMATION_TECHNOLOGY_SECTOR,
                             NUCLEAR_REACTORS_MATERIALS_AND_WASTE_SECTOR,
                             TRANSPORTATION_SYSTEMS_SECTOR,
                             WATER_AND_WASTEWATER_SYSTEMS_SECTOR)

    if isinstance(industry_type, str):
        if industry_type.split(" | ") > 1:
            val = industry_type.split(" | ")

            # Pipe-delimited string supplied.
            if all(x in INDUSTRY_TYPE_SECTORS for x in val):
                return industry_type

        # Single string supplied.
        if industry_type in INDUSTRY_TYPE_SECTORS:
            return industry_type

    elif isinstance(industry_type, list):
        # Create pipe-delimited string when list of strings is provided.
        if all(x in INDUSTRY_TYPE_SECTORS for x in industry_type):
            return " | ".join(industry_type)

    msg = 'IndustryType must be one of the following: {0}. Received \'{1}\'.'
    raise ValueError(msg.format(INDUSTRY_TYPE_SECTORS, industry_type))


def add_ais_marking(stix_package, proprietary, consent, color, **kwargs):
    """
    This utility functions aids in the creation of an AIS marking and appends
    it to the provided STIX package.

    Args:
        stix_package: A stix.core.STIXPackage object.
        proprietary: True if marking uses IsProprietary, False for
        NotProprietary.
        consent: A string with one of the following values: "EVERYONE", "NONE"
            or "USG".
        color: A string that corresponds to TLP values: "WHITE", "GREEN" or
            "AMBER".
        **kwargs: Six required keyword arguments that are used to create a CIQ
            identity object. These are: country_name_code,
            country_name_code_type, admin_area_name_code,
            admin_area_name_code_type, organisation_name, industry_type.

    Raises:
        ValueError: When keyword arguments are missing. User did not supply
            correct values for: proprietary, color and consent.

    Note:
        Any Markings under STIX Header will be removed. Please follow the
        guidelines for `AIS`_.

        The industry_type keyword argument accepts: a list of string based on
        defined sectors, a pipe-delimited string of sectors, or a single
        sector.

    .. _AIS:
        https://www.us-cert.gov/ais
    """
    from stix.common import InformationSource
    from stix.extensions.identity.ciq_identity_3_0 import (
        CIQIdentity3_0Instance, STIXCIQIdentity3_0, PartyName, Address,
        Country, NameElement, OrganisationInfo, AdministrativeArea)
    from stix.core.stix_header import STIXHeader
    from stix.data_marking import MarkingSpecification, Marking

    args = ('country_name_code', 'country_name_code_type', 'industry_type',
            'admin_area_name_code', 'admin_area_name_code_type',
            'organisation_name')

    diff = set(args) - set(kwargs.keys())

    if diff:
        msg = 'All keyword arguments must be provided. Missing: {0}'
        raise ValueError(msg.format(tuple(diff)))

    party_name = PartyName()
    party_name.add_organisation_name(kwargs['organisation_name'])

    country = Country()
    country_name = NameElement()
    country_name.name_code = kwargs['country_name_code']
    country_name.name_code_type = kwargs['country_name_code_type']
    country.add_name_element(country_name)

    admin_area = AdministrativeArea()
    admin_area_name = NameElement()
    admin_area_name.name_code = kwargs['admin_area_name_code']
    admin_area_name.name_code_type = kwargs['admin_area_name_code_type']
    admin_area.add_name_element(admin_area_name)

    address = Address()
    address.country = country
    address.administrative_area = admin_area

    org_info = OrganisationInfo()
    org_info.industry_type = _validate_and_create_industry_type(kwargs['industry_type'])

    id_spec = STIXCIQIdentity3_0()
    id_spec.party_name = party_name
    id_spec.add_address(address)
    id_spec.organisation_info = org_info

    identity = CIQIdentity3_0Instance()
    identity.specification = id_spec

    if proprietary is True:
        proprietary_obj = IsProprietary()
        consent = 'EVERYONE'
    elif proprietary is False:
        proprietary_obj = NotProprietary()
    else:
        raise ValueError('proprietary expected True or False.')

    proprietary_obj.ais_consent = AISConsentType(consent=consent)
    proprietary_obj.tlp_marking = TLPMarkingType(color=color)

    ais_marking = AISMarkingStructure()

    if isinstance(proprietary_obj, IsProprietary):
        ais_marking.is_proprietary = proprietary_obj
    else:
        ais_marking.not_proprietary = proprietary_obj

    marking_spec = MarkingSpecification()
    marking_spec.controlled_structure = '//node() | //@*'
    marking_spec.marking_structures.append(ais_marking)
    marking_spec.information_source = InformationSource()
    marking_spec.information_source.identity = identity

    if not stix_package.stix_header:
        stix_package.stix_header = STIXHeader()

    # Removes any other Markings if present.
    stix_package.stix_header.handling = Marking()
    stix_package.stix_header.handling.add_marking(marking_spec)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# external
from cybox.core import Observables

# internal
import stix
from stix.common import StructuredText, VocabString
from stix.common.vocabs import AttackerInfrastructureType
import stix.bindings.ttp as ttp_binding


class Infrastructure(stix.Entity):
    _binding = ttp_binding
    _binding_class = _binding.InfrastructureType
    _namespace = "http://stix.mitre.org/TTP-1"

    def __init__(self, id_=None, title=None, description=None, short_description=None):
        self.id_ = id_
        self.title = title
        self.description = description
        self.short_description = short_description
        self.types = None
        self.observable_characterization = None

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = value

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        self._set_var(StructuredText, description=value)

    @property
    def short_description(self):
        return self._short_description

    @short_description.setter
    def short_description(self, value):
        self._set_var(StructuredText, short_description=value)

    @property
    def types(self):
        return self._types

    @types.setter
    def types(self, value):
        self._types = InfraStructureTypes(value)

    def add_type(self, type_):
        self.types.append(type_)

    @property
    def observable_characterization(self):
        return self._observable_characterization

    @observable_characterization.setter
    def observable_characterization(self, value):
        self._set_var(Observables, observable_characterization=value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(Infrastructure, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        return_obj.id = self.id_
        return_obj.Title = self.title

        if self.description:
            return_obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.short_description:
            return_obj.Short_Description = self.short_description.to_obj(ns_info=ns_info)
        if self.types:
            return_obj.Type = [x.to_obj(ns_info=ns_info) for x in self.types]
        if self.observable_characterization:
            return_obj.Observable_Characterization = self.observable_characterization.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.id_ = obj.id
        return_obj.title = obj.Title
        return_obj.description = StructuredText.from_obj(obj.Description)
        return_obj.short_description = StructuredText.from_obj(obj.Short_Description)
        return_obj.observable_characterization = Observables.from_obj(obj.Observable_Characterization)

        if obj.Type:
            return_obj.types = [VocabString.from_obj(x) for x in obj.Type]

        return return_obj

    def to_dict(self):
        return super(Infrastructure, self).to_dict()

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.id_ = dict_repr.get('id')
        return_obj.title = dict_repr.get('title')
        return_obj.description = StructuredText.from_dict(dict_repr.get('description'))
        return_obj.short_description = StructuredText.from_dict(dict_repr.get('short_description'))
        return_obj.types = [VocabString.from_dict(x) for x in dict_repr.get('types', [])]
        return_obj.observable_characterization = Observables.from_dict(dict_repr.get('observable_characterization'))

        return return_obj


class InfraStructureTypes(stix.EntityList):
    _namespace = "http://stix.mitre.org/TTP-1"
    _contained_type = VocabString
    _dict_as_list = True

    def _fix_value(self, value):
        return AttackerInfrastructureType(value)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from stix.test import EntityTestCase, TypedListTestCase, data_marking_test
from stix.test.common import (
    confidence_test, information_source_test, statement_test, related_test,
    activity_test
)

import stix.campaign as campaign


class NamesTests(EntityTestCase, unittest.TestCase):
    klass = campaign.Names

    _full_dict = {
        'names': [
            "Dancing Hippos",
            "Crazy Squirrels",
            {
                'value': "Medium",
                'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'
            }
        ]
    }

class IntendedEffectsTests(TypedListTestCase, unittest.TestCase):
    klass = campaign._IntendedEffects

    _full_dict = [
        statement_test.StatementTests._full_dict
    ]


class RelatedTTPsTest(EntityTestCase, unittest.TestCase):
    klass = campaign.RelatedTTPs

    _full_dict = {
        'scope': 'exclusive',
        'ttps': [
            related_test.RelatedTTPTests._full_dict,
        ]
    }


class RelatedIncidentsTests(EntityTestCase, unittest.TestCase):
    klass = campaign.RelatedIncidents

    _full_dict = {
        'incidents': [
            related_test.RelatedIncidentTests._full_dict
        ]
    }


class RelatedIndicatorsTests(EntityTestCase, unittest.TestCase):
    klass = campaign.RelatedIndicators

    _full_dict = {
        'indicators': [
            related_test.RelatedIndicatorTests._full_dict
        ]
    }

class AttributionTests(EntityTestCase, unittest.TestCase):
    klass = campaign.Attribution

    _full_dict = {
        'scope': 'exclusive',
        'threat_actors': [
            related_test.RelatedThreatActorTests._full_dict,
        ]
    }


class AttributionListTests(TypedListTestCase, unittest.TestCase):
    klass = campaign._AttributionList

    _full_dict = [
        AttributionTests._full_dict
    ]


class AssociatedCampaignsTests(EntityTestCase, unittest.TestCase):
    klass = campaign.AssociatedCampaigns

    _full_dict = {
        'scope': 'inclusive',
        'campaigns': [
            related_test.RelatedCampaignTests._full_dict
        ]
    }


class ActivitiesTests(TypedListTestCase, unittest.TestCase):
    klass = campaign._Activities

    _full_dict = [
        activity_test.ActivityTests._full_dict
    ]


class CampaignTest(EntityTestCase, unittest.TestCase):
    klass = campaign.Campaign
    _full_dict = {
        'id': "example:test-1",
        'timestamp': "2014-01-31T06:14:46",
        'version': '1.1.1',
        'title': 'Purple Elephant',
        'description': 'A pretty novice set of actors.',
        'short_description': 'novices',
        'names': NamesTests._full_dict,
        'intended_effects': IntendedEffectsTests._full_dict,
        'status': {
            'value': "Ongoing",
            'xsi:type':'stixVocabs:CampaignStatusVocab-1.0'
        },
        'related_ttps': RelatedTTPsTest._full_dict,
        'related_incidents': RelatedIncidentsTests._full_dict,
        'related_indicators': RelatedIndicatorsTests._full_dict,
        'attribution': AttributionListTests._full_dict,
        'associated_campaigns': AssociatedCampaignsTests._full_dict,
        'confidence': confidence_test.ConfidenceTests._full_dict,
        'activity': ActivitiesTests._full_dict,
        'information_source': information_source_test.InformationSourceTests._full_dict,
        'handling': data_marking_test.MarkingTests._full_dict,
        'related_packages': related_test.RelatedPackageRefsTests._full_dict
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.bindings.stix_common as stix_common_binding


# TODO: handle normalization
# from cybox.utils import normalize_to_xml, denormalize_from_xml


class VocabString(stix.Entity):
    _binding = stix_common_binding
    _binding_class = stix_common_binding.ControlledVocabularyStringType
    _namespace = 'http://stix.mitre.org/common-1'

    # All subclasses should override these
    _XSI_TYPE = None
    _ALLOWED_VALUES = None

    def __init__(self, value=None):
        super(VocabString, self).__init__()
        self.value = value
        self.xsi_type = self._XSI_TYPE

        self.vocab_name = None
        self.vocab_reference = None

    @property
    def value(self):
        return self._value
    
    @value.setter
    def value(self, v):
        allowed = self._ALLOWED_VALUES

        if not v:
            self._value = None
        elif allowed and (v not in allowed):
            error = "Value must be one of {0}. Received '{1}'"
            error = error.format(allowed, v)
            raise ValueError(error)
        else:
            self._value = v

    def __str__(self):
        return str(self.value)

    def __eq__(self, other):
        # Check to make sure the values are identical.
        if isinstance(other, VocabString):
            other = other.value

        return other == self.value

    def is_plain(self):
        """Whether the VocabString can be represented as a single value."""
        return (
            self._XSI_TYPE is None and
            self.vocab_name is None and
            self.vocab_reference is None
        )

    @staticmethod
    def lookup_class(xsi_type):
        if not xsi_type:
            return VocabString
        
        for (k, v) in _VOCAB_MAP.iteritems():
            # TODO: for now we ignore the prefix and just check for
            # a partial match
            if xsi_type in k:
                return v

        return VocabString

    def to_obj(self, return_obj=None, ns_info=None):
        super(VocabString, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        # TODO: handle normalization
        # vocab_obj.valueOf_ = normalize_to_xml(self.value)
        return_obj.valueOf_ = self.value
        return_obj.xsi_type = self.xsi_type

        if self.vocab_name is not None:
            return_obj.vocab_name = self.vocab_name
        if self.vocab_reference is not None:
            return_obj.vocab_reference = self.vocab_reference

        return return_obj

    def to_dict(self):
        if self.is_plain():
            return self.value

        d = {}
        if self.value is not None:
            d['value'] = self.value
        if self.xsi_type is not None:
            d['xsi:type'] = self.xsi_type
        if self.vocab_name is not None:
            d['vocab_name'] = self.vocab_name
        if self.vocab_reference is not None:
            d['vocab_reference'] = self.vocab_reference

        return d

    @classmethod
    def from_obj(cls, vocab_obj, return_obj=None):
        if not vocab_obj:
            return None
        
        if not return_obj:
            klass = VocabString.lookup_class(vocab_obj.xsi_type)
            return klass.from_obj(vocab_obj, klass())
           
        # xsi_type should be set automatically by the class's constructor.
        
        # TODO: handle denormalization
        # vocab_str.value = denormalize_from_xml(vocab_obj.valueOf_)
        return_obj.value = vocab_obj.valueOf_
        return_obj.vocab_name = vocab_obj.vocab_name
        return_obj.vocab_reference = vocab_obj.vocab_reference
        return_obj.xsi_type = vocab_obj.xsi_type

        return return_obj

    @classmethod
    def from_dict(cls, vocab_dict, return_obj=None):
        if not vocab_dict:
            return None
        
        if not return_obj:
            if isinstance(vocab_dict, dict):
                klass = VocabString.lookup_class(vocab_dict.get('xsi:type'))
                return klass.from_dict(vocab_dict, klass())
            else:
                return_obj = cls()
            
        # xsi_type should be set automatically by the class's constructor.

        # In case this is a "plain" string, just set it.
        if not isinstance(vocab_dict, dict):
            return_obj.value = vocab_dict
        else:
            return_obj.value = vocab_dict.get('value')
            return_obj.vocab_name = vocab_dict.get('vocab_name')
            return_obj.vocab_reference = vocab_dict.get('vocab_reference')
            return_obj.xsi_type = vocab_dict.get('xsi:type')

        return return_obj


#: Mapping of Controlled Vocabulary xsi:type's to their class implementations.
_VOCAB_MAP = {}


def _get_terms(vocab_class):
    """Helper function used by register_vocab."""
    for k, v in vocab_class.__dict__.items():
        if k.startswith("TERM_"):
            yield v


def add_vocab(cls):
    """Registers a VocabString subclass.

    Note:
        The :meth:`register_vocab` class decorator has replaced this method.

    """
    _VOCAB_MAP[cls._XSI_TYPE] = cls


def register_vocab(cls):
    """Register a VocabString subclass.

    Also, calculate all the permitted values for class being decorated by
    adding an ``_ALLOWED_VALUES`` tuple of all the values of class members
    beginning with ``TERM_``.

    """
    add_vocab(cls)

    cls._ALLOWED_VALUES = tuple(_get_terms(cls))
    return cls


@register_vocab
class AvailabilityLossType(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:AvailabilityLossTypeVocab-1.1.1'

    TERM_DESTRUCTION = "Destruction"
    TERM_LOSS = "Loss"
    TERM_INTERRUPTION = "Interruption"
    TERM_DEGRADATION = "Degradation"
    TERM_ACCELERATION = "Acceleration"
    TERM_OBSCURATION = "Obscuration"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class ThreatActorType(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:ThreatActorTypeVocab-1.0'

    TERM_CYBER_ESPIONAGE_OPERATIONS = "Cyber Espionage Operations"
    TERM_HACKER = "Hacker"
    TERM_HACKER_WHITE_HAT = "Hacker - White hat"
    TERM_HACKER_GRAY_HAT = "Hacker - Gray hat"
    TERM_HACKER_BLACK_HAT = "Hacker - Black hat"
    TERM_HACKTIVIST = "Hacktivist"
    TERM_STATE_ACTOR_OR_AGENCY = "State Actor / Agency"
    TERM_ECRIME_ACTOR_CREDENTIAL_THEFT_BOTNET_OPERATOR = "eCrime Actor - Credential Theft Botnet Operator"
    TERM_ECRIME_ACTOR_CREDENTIAL_THEFT_BOTNET_SERVICE = "eCrime Actor - Credential Theft Botnet Service"
    TERM_ECRIME_ACTOR_MALWARE_DEVELOPER = "eCrime Actor - Malware Developer"
    TERM_ECRIME_ACTOR_MONEY_LAUNDERING_NETWORK = "eCrime Actor - Money Laundering Network"
    TERM_ECRIME_ACTOR_ORGANIZED_CRIME_ACTOR = "eCrime Actor - Organized Crime Actor"
    TERM_ECRIME_ACTOR_SPAM_SERVICE = "eCrime Actor - Spam Service"
    TERM_ECRIME_ACTOR_TRAFFIC_SERVICE = "eCrime Actor - Traffic Service"
    TERM_ECRIME_ACTOR_UNDERGROUND_CALL_SERVICE = "eCrime Actor - Underground Call Service"
    TERM_INSIDER_THREAT = "Insider Threat"
    TERM_DISGRUNTLED_CUSTOMER_OR_USER = "Disgruntled Customer / User"


@register_vocab
class AttackerInfrastructureType(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:AttackerInfrastructureTypeVocab-1.0'

    TERM_ANONYMIZATION = "Anonymization"
    TERM_ANONYMIZATION_PROXY = "Anonymization - Proxy"
    TERM_ANONYMIZATION_TOR_NETWORK = "Anonymization - TOR Network"
    TERM_ANONYMIZATION_VPN = "Anonymization - VPN"
    TERM_COMMUNICATIONS = "Communications"
    TERM_COMMUNICATIONS_BLOGS = "Communications - Blogs"
    TERM_COMMUNICATIONS_FORUMS = "Communications - Forums"
    TERM_COMMUNICATIONS_INTERNET_RELAY_CHAT = "Communications - Internet Relay Chat"
    TERM_COMMUNICATIONS_MICROBLOGS = "Communications - Micro-Blogs"
    TERM_COMMUNICATIONS_MOBILE_COMMUNICATIONS = "Communications - Mobile Communications"
    TERM_COMMUNICATIONS_SOCIAL_NETWORKS = "Communications - Social Networks"
    TERM_COMMUNICATIONS_USERGENERATED_CONTENT_WEBSITES = "Communications - User-Generated Content Websites"
    TERM_DOMAIN_REGISTRATION = "Domain Registration"
    TERM_DOMAIN_REGISTRATION_DYNAMIC_DNS_SERVICES = "Domain Registration - Dynamic DNS Services"
    TERM_DOMAIN_REGISTRATION_LEGITIMATE_DOMAIN_REGISTRATION_SERVICES = "Domain Registration - Legitimate Domain Registration Services"
    TERM_DOMAIN_REGISTRATION_MALICIOUS_DOMAIN_REGISTRARS = "Domain Registration - Malicious Domain Registrars"
    TERM_DOMAIN_REGISTRATION_TOPLEVEL_DOMAIN_REGISTRARS = "Domain Registration - Top-Level Domain Registrars"
    TERM_HOSTING = "Hosting"
    TERM_HOSTING_BULLETPROOF_OR_ROGUE_HOSTING = "Hosting - Bulletproof / Rogue Hosting"
    TERM_HOSTING_CLOUD_HOSTING = "Hosting - Cloud Hosting"
    TERM_HOSTING_COMPROMISED_SERVER = "Hosting - Compromised Server"
    TERM_HOSTING_FAST_FLUX_BOTNET_HOSTING = "Hosting - Fast Flux Botnet Hosting"
    TERM_HOSTING_LEGITIMATE_HOSTING = "Hosting - Legitimate Hosting"
    TERM_ELECTRONIC_PAYMENT_METHODS = "Electronic Payment Methods"


@register_vocab
class DiscoveryMethod(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:DiscoveryMethodVocab-1.0'

    TERM_AGENT_DISCLOSURE = "Agent Disclosure"
    TERM_FRAUD_DETECTION = "Fraud Detection"
    TERM_MONITORING_SERVICE = "Monitoring Service"
    TERM_LAW_ENFORCEMENT = "Law Enforcement"
    TERM_CUSTOMER = "Customer"
    TERM_UNRELATED_PARTY = "Unrelated Party"
    TERM_AUDIT = "Audit"
    TERM_ANTIVIRUS = "Antivirus"
    TERM_INCIDENT_RESPONSE = "Incident Response"
    TERM_FINANCIAL_AUDIT = "Financial Audit"
    TERM_HIPS = "HIPS"
    TERM_IT_AUDIT = "IT Audit"
    TERM_LOG_REVIEW = "Log Review"
    TERM_NIDS = "NIDS"
    TERM_SECURITY_ALARM = "Security Alarm"
    TERM_USER = "User"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class AttackerToolType(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:AttackerToolTypeVocab-1.0'

    TERM_MALWARE = "Malware"
    TERM_PENETRATION_TESTING = "Penetration Testing"
    TERM_PORT_SCANNER = "Port Scanner"
    TERM_TRAFFIC_SCANNER = "Traffic Scanner"
    TERM_VULNERABILITY_SCANNER = "Vulnerability Scanner"
    TERM_APPLICATION_SCANNER = "Application Scanner"
    TERM_PASSWORD_CRACKING = "Password Cracking"


@register_vocab
class IndicatorType(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:IndicatorTypeVocab-1.1'

    TERM_MALICIOUS_EMAIL = "Malicious E-mail"
    TERM_IP_WATCHLIST = "IP Watchlist"
    TERM_FILE_HASH_WATCHLIST = "File Hash Watchlist"
    TERM_DOMAIN_WATCHLIST = "Domain Watchlist"
    TERM_URL_WATCHLIST = "URL Watchlist"
    TERM_MALWARE_ARTIFACTS = "Malware Artifacts"
    TERM_C2 = "C2"
    TERM_ANONYMIZATION = "Anonymization"
    TERM_EXFILTRATION = "Exfiltration"
    TERM_HOST_CHARACTERISTICS = "Host Characteristics"
    TERM_COMPROMISED_PKI_CERTIFICATE = "Compromised PKI Certificate"
    TERM_LOGIN_NAME = "Login Name"
    TERM_IMEI_WATCHLIST = "IMEI Watchlist"
    TERM_IMSI_WATCHLIST = "IMSI Watchlist"


@register_vocab
class SystemType(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:SystemTypeVocab-1.0'

    TERM_ENTERPRISE_SYSTEMS = "Enterprise Systems"
    TERM_ENTERPRISE_SYSTEMS_APPLICATION_LAYER = "Enterprise Systems - Application Layer"
    TERM_ENTERPRISE_SYSTEMS_DATABASE_LAYER = "Enterprise Systems - Database Layer"
    TERM_ENTERPRISE_SYSTEMS_ENTERPRISE_TECHNOLOGIES_AND_SUPPORT_INFRASTRUCTURE = "Enterprise Systems - Enterprise Technologies and Support Infrastructure"
    TERM_ENTERPRISE_SYSTEMS_NETWORK_SYSTEMS = "Enterprise Systems - Network Systems"
    TERM_ENTERPRISE_SYSTEMS_NETWORKING_DEVICES = "Enterprise Systems - Networking Devices"
    TERM_ENTERPRISE_SYSTEMS_WEB_LAYER = "Enterprise Systems - Web Layer"
    TERM_ENTERPRISE_SYSTEMS_VOIP = "Enterprise Systems - VoIP"
    TERM_INDUSTRIAL_CONTROL_SYSTEMS = "Industrial Control Systems"
    TERM_INDUSTRIAL_CONTROL_SYSTEMS_EQUIPMENT_UNDER_CONTROL = "Industrial Control Systems - Equipment Under Control"
    TERM_INDUSTRIAL_CONTROL_SYSTEMS_OPERATIONS_MANAGEMENT = "Industrial Control Systems - Operations Management"
    TERM_INDUSTRIAL_CONTROL_SYSTEMS_SAFETY_PROTECTION_AND_LOCAL_CONTROL = "Industrial Control Systems - Safety, Protection and Local Control"
    TERM_INDUSTRIAL_CONTROL_SYSTEMS_SUPERVISORY_CONTROL = "Industrial Control Systems - Supervisory Control"
    TERM_MOBILE_SYSTEMS = "Mobile Systems"
    TERM_MOBILE_SYSTEMS_MOBILE_OPERATING_SYSTEMS = "Mobile Systems - Mobile Operating Systems"
    TERM_MOBILE_SYSTEMS_NEAR_FIELD_COMMUNICATIONS = "Mobile Systems - Near Field Communications"
    TERM_MOBILE_SYSTEMS_MOBILE_DEVICES = "Mobile Systems - Mobile Devices"
    TERM_THIRDPARTY_SERVICES = "Third-Party Services"
    TERM_THIRDPARTY_SERVICES_APPLICATION_STORES = "Third-Party Services - Application Stores"
    TERM_THIRDPARTY_SERVICES_CLOUD_SERVICES = "Third-Party Services - Cloud Services"
    TERM_THIRDPARTY_SERVICES_SECURITY_VENDORS = "Third-Party Services - Security Vendors"
    TERM_THIRDPARTY_SERVICES_SOCIAL_MEDIA = "Third-Party Services - Social Media"
    TERM_THIRDPARTY_SERVICES_SOFTWARE_UPDATE = "Third-Party Services - Software Update"
    TERM_USERS = "Users"
    TERM_USERS_APPLICATION_AND_SOFTWARE = "Users - Application And Software"
    TERM_USERS_WORKSTATION = "Users - Workstation"
    TERM_USERS_REMOVABLE_MEDIA = "Users - Removable Media"


@register_vocab
class CampaignStatus(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:CampaignStatusVocab-1.0'

    TERM_ONGOING = "Ongoing"
    TERM_HISTORIC = "Historic"
    TERM_FUTURE = "Future"


@register_vocab
class IncidentStatus(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:IncidentStatusVocab-1.0'

    TERM_NEW = "New"
    TERM_OPEN = "Open"
    TERM_STALLED = "Stalled"
    TERM_CONTAINMENT_ACHIEVED = "Containment Achieved"
    TERM_RESTORATION_ACHIEVED = "Restoration Achieved"
    TERM_INCIDENT_REPORTED = "Incident Reported"
    TERM_CLOSED = "Closed"
    TERM_REJECTED = "Rejected"
    TERM_DELETED = "Deleted"


@register_vocab
class ManagementClass(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:ManagementClassVocab-1.0'

    TERM_INTERNALLYMANAGED = "Internally-Managed"
    TERM_EXTERNALLYMANAGEMENT = "Externally-Management"
    TERM_COMANAGEMENT = "Co-Management"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class Motivation(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:MotivationVocab-1.1'

    TERM_IDEOLOGICAL = "Ideological"
    TERM_IDEOLOGICAL_ANTICORRUPTION = "Ideological - Anti-Corruption"
    TERM_IDEOLOGICAL_ANTIESTABLISHMENT = "Ideological - Anti-Establishment"
    TERM_IDEOLOGICAL_ENVIRONMENTAL = "Ideological - Environmental"
    TERM_IDEOLOGICAL_ETHNIC_OR_NATIONALIST = "Ideological - Ethnic / Nationalist"
    TERM_IDEOLOGICAL_INFORMATION_FREEDOM = "Ideological - Information Freedom"
    TERM_IDEOLOGICAL_RELIGIOUS = "Ideological - Religious"
    TERM_IDEOLOGICAL_SECURITY_AWARENESS = "Ideological - Security Awareness"
    TERM_IDEOLOGICAL_HUMAN_RIGHTS = "Ideological - Human Rights"
    TERM_EGO = "Ego"
    TERM_FINANCIAL_OR_ECONOMIC = "Financial or Economic"
    TERM_MILITARY = "Military"
    TERM_OPPORTUNISTIC = "Opportunistic"
    TERM_POLITICAL = "Political"


@register_vocab
class IncidentCategory(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:IncidentCategoryVocab-1.0'

    TERM_EXERCISEORNETWORK_DEFENSE_TESTING = "Exercise/Network Defense Testing"
    TERM_UNAUTHORIZED_ACCESS = "Unauthorized Access"
    TERM_DENIAL_OF_SERVICE = "Denial of Service"
    TERM_MALICIOUS_CODE = "Malicious Code"
    TERM_IMPROPER_USAGE = "Improper Usage"
    TERM_SCANSORPROBESORATTEMPTED_ACCESS = "Scans/Probes/Attempted Access"
    TERM_INVESTIGATION = "Investigation"


@register_vocab
class ImpactQualification(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:ImpactQualificationVocab-1.0'

    TERM_INSIGNIFICANT = "Insignificant"
    TERM_DISTRACTING = "Distracting"
    TERM_PAINFUL = "Painful"
    TERM_DAMAGING = "Damaging"
    TERM_CATASTROPHIC = "Catastrophic"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class PlanningAndOperationalSupport(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:PlanningAndOperationalSupportVocab-1.0.1'

    TERM_DATA_EXPLOITATION = "Data Exploitation"
    TERM_DATA_EXPLOITATION_ANALYTIC_SUPPORT = "Data Exploitation - Analytic Support"
    TERM_DATA_EXPLOITATION_TRANSLATION_SUPPORT = "Data Exploitation - Translation Support"
    TERM_FINANCIAL_RESOURCES = "Financial Resources"
    TERM_FINANCIAL_RESOURCES_ACADEMIC = "Financial Resources - Academic"
    TERM_FINANCIAL_RESOURCES_COMMERCIAL = "Financial Resources - Commercial"
    TERM_FINANCIAL_RESOURCES_GOVERNMENT = "Financial Resources - Government"
    TERM_FINANCIAL_RESOURCES_HACKTIVIST_OR_GRASSROOT = "Financial Resources - Hacktivist or Grassroot"
    TERM_FINANCIAL_RESOURCES_NONATTRIBUTABLE_FINANCE = "Financial Resources - Non-Attributable Finance"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT = "Skill Development / Recruitment"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_CONTRACTING_AND_HIRING = "Skill Development / Recruitment - Contracting and Hiring"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_DOCUMENT_EXPLOITATION_DOCEX_TRAINING = "Skill Development / Recruitment - Document Exploitation (DOCEX) Training"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_INTERNAL_TRAINING = "Skill Development / Recruitment - Internal Training"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_MILITARY_PROGRAMS = "Skill Development / Recruitment - Military Programs"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_SECURITY_OR_HACKER_CONFERENCES = "Skill Development / Recruitment - Security / Hacker Conferences"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_UNDERGROUND_FORUMS = "Skill Development / Recruitment - Underground Forums"
    TERM_SKILL_DEVELOPMENT_OR_RECRUITMENT_UNIVERSITY_PROGRAMS = "Skill Development / Recruitment - University Programs"
    TERM_PLANNING = "Planning"
    TERM_PLANNING_OPERATIONAL_COVER_PLAN = "Planning - Operational Cover Plan"
    TERM_PLANNING_OPENSOURCE_INTELLIGENCE_OSINT_GATHERING = "Planning - Open-Source Intelligence (OSINT) Gathering"
    TERM_PLANNING_PREOPERATIONAL_SURVEILLANCE_AND_RECONNAISSANCE = "Planning - Pre-Operational Surveillance and Reconnaissance"
    TERM_PLANNING_TARGET_SELECTION = "Planning - Target Selection"


@register_vocab
class CourseOfActionType(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:CourseOfActionTypeVocab-1.0'

    TERM_PERIMETER_BLOCKING = "Perimeter Blocking"
    TERM_INTERNAL_BLOCKING = "Internal Blocking"
    TERM_REDIRECTION = "Redirection"
    TERM_REDIRECTION_HONEY_POT = "Redirection (Honey Pot)"
    TERM_HARDENING = "Hardening"
    TERM_PATCHING = "Patching"
    TERM_ERADICATION = "Eradication"
    TERM_REBUILDING = "Rebuilding"
    TERM_TRAINING = "Training"
    TERM_MONITORING = "Monitoring"
    TERM_PHYSICAL_ACCESS_RESTRICTIONS = "Physical Access Restrictions"
    TERM_LOGICAL_ACCESS_RESTRICTIONS = "Logical Access Restrictions"
    TERM_PUBLIC_DISCLOSURE = "Public Disclosure"
    TERM_DIPLOMATIC_ACTIONS = "Diplomatic Actions"
    TERM_POLICY_ACTIONS = "Policy Actions"
    TERM_OTHER = "Other"


@register_vocab
class SecurityCompromise(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:SecurityCompromiseVocab-1.0'

    TERM_YES = "Yes"
    TERM_SUSPECTED = "Suspected"
    TERM_NO = "No"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class ImpactRating(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:ImpactRatingVocab-1.0'

    TERM_NONE = "None"
    TERM_MINOR = "Minor"
    TERM_MODERATE = "Moderate"
    TERM_MAJOR = "Major"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class AssetType(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:AssetTypeVocab-1.0'

    TERM_BACKUP = "Backup"
    TERM_DATABASE = "Database"
    TERM_DHCP = "DHCP"
    TERM_DIRECTORY = "Directory"
    TERM_DCS = "DCS"
    TERM_DNS = "DNS"
    TERM_FILE = "File"
    TERM_LOG = "Log"
    TERM_MAIL = "Mail"
    TERM_MAINFRAME = "Mainframe"
    TERM_PAYMENT_SWITCH = "Payment switch"
    TERM_POS_CONTROLLER = "POS controller"
    TERM_PRINT = "Print"
    TERM_PROXY = "Proxy"
    TERM_REMOTE_ACCESS = "Remote access"
    TERM_SCADA = "SCADA"
    TERM_WEB_APPLICATION = "Web application"
    TERM_SERVER = "Server"
    TERM_ACCESS_READER = "Access reader"
    TERM_CAMERA = "Camera"
    TERM_FIREWALL = "Firewall"
    TERM_HSM = "HSM"
    TERM_IDS = "IDS"
    TERM_BROADBAND = "Broadband"
    TERM_PBX = "PBX"
    TERM_PRIVATE_WAN = "Private WAN"
    TERM_PLC = "PLC"
    TERM_PUBLIC_WAN = "Public WAN"
    TERM_RTU = "RTU"
    TERM_ROUTER_OR_SWITCH = "Router or switch"
    TERM_SAN = "SAN"
    TERM_TELEPHONE = "Telephone"
    TERM_VOIP_ADAPTER = "VoIP adapter"
    TERM_LAN = "LAN"
    TERM_WLAN = "WLAN"
    TERM_NETWORK = "Network"
    TERM_AUTH_TOKEN = "Auth token"
    TERM_ATM = "ATM"
    TERM_DESKTOP = "Desktop"
    TERM_PED_PAD = "PED pad"
    TERM_GAS_TERMINAL = "Gas terminal"
    TERM_LAPTOP = "Laptop"
    TERM_MEDIA = "Media"
    TERM_MOBILE_PHONE = "Mobile phone"
    TERM_PERIPHERAL = "Peripheral"
    TERM_POS_TERMINAL = "POS terminal"
    TERM_KIOSK = "Kiosk"
    TERM_TABLET = "Tablet"
    TERM_VOIP_PHONE = "VoIP phone"
    TERM_USER_DEVICE = "User Device"
    TERM_TAPES = "Tapes"
    TERM_DISK_MEDIA = "Disk media"
    TERM_DOCUMENTS = "Documents"
    TERM_FLASH_DRIVE = "Flash drive"
    TERM_DISK_DRIVE = "Disk drive"
    TERM_SMART_CARD = "Smart card"
    TERM_PAYMENT_CARD = "Payment card"
    TERM_ADMINISTRATOR = "Administrator"
    TERM_AUDITOR = "Auditor"
    TERM_CALL_CENTER = "Call center"
    TERM_CASHIER = "Cashier"
    TERM_CUSTOMER = "Customer"
    TERM_DEVELOPER = "Developer"
    TERM_ENDUSER = "End-user"
    TERM_EXECUTIVE = "Executive"
    TERM_FINANCE = "Finance"
    TERM_FORMER_EMPLOYEE = "Former employee"
    TERM_GUARD = "Guard"
    TERM_HELPDESK = "Helpdesk"
    TERM_HUMAN_RESOURCES = "Human resources"
    TERM_MAINTENANCE = "Maintenance"
    TERM_MANAGER = "Manager"
    TERM_PARTNER = "Partner"
    TERM_PERSON = "Person"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class COAStage(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:COAStageVocab-1.0'

    TERM_REMEDY = "Remedy"
    TERM_RESPONSE = "Response"


@register_vocab
class LocationClass(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:LocationClassVocab-1.0'

    TERM_INTERNALLYLOCATED = "Internally-Located"
    TERM_EXTERNALLYLOCATED = "Externally-Located"
    TERM_COLOCATED = "Co-Located"
    TERM_MOBILE = "Mobile"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class InformationType(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:InformationTypeVocab-1.0'

    TERM_INFORMATION_ASSETS = "Information Assets"
    TERM_INFORMATION_ASSETS_CORPORATE_EMPLOYEE_INFORMATION = "Information Assets - Corporate Employee Information"
    TERM_INFORMATION_ASSETS_CUSTOMER_PII = "Information Assets - Customer PII"
    TERM_INFORMATION_ASSETS_EMAIL_LISTS_OR_ARCHIVES = "Information Assets - Email Lists / Archives"
    TERM_INFORMATION_ASSETS_FINANCIAL_DATA = "Information Assets - Financial Data"
    TERM_INFORMATION_ASSETS_INTELLECTUAL_PROPERTY = "Information Assets - Intellectual Property"
    TERM_INFORMATION_ASSETS_MOBILE_PHONE_CONTACTS = "Information Assets - Mobile Phone Contacts"
    TERM_INFORMATION_ASSETS_USER_CREDENTIALS = "Information Assets - User Credentials"
    TERM_AUTHENTICATION_COOKIES = "Authentication Cookies"


@register_vocab
class ThreatActorSophistication(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:ThreatActorSophisticationVocab-1.0'

    TERM_INNOVATOR = "Innovator"
    TERM_EXPERT = "Expert"
    TERM_PRACTITIONER = "Practitioner"
    TERM_NOVICE = "Novice"
    TERM_ASPIRANT = "Aspirant"


@register_vocab
class HighMediumLow(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:HighMediumLowVocab-1.0'

    TERM_HIGH = "High"
    TERM_MEDIUM = "Medium"
    TERM_LOW = "Low"
    TERM_NONE = "None"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class LossProperty(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:LossPropertyVocab-1.0'

    TERM_CONFIDENTIALITY = "Confidentiality"
    TERM_INTEGRITY = "Integrity"
    TERM_AVAILABILITY = "Availability"
    TERM_ACCOUNTABILITY = "Accountability"
    TERM_NONREPUDIATION = "Non-Repudiation"


@register_vocab
class IntendedEffect(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:IntendedEffectVocab-1.0'

    TERM_ADVANTAGE = "Advantage"
    TERM_ADVANTAGE_ECONOMIC = "Advantage - Economic"
    TERM_ADVANTAGE_MILITARY = "Advantage - Military"
    TERM_ADVANTAGE_POLITICAL = "Advantage - Political"
    TERM_THEFT = "Theft"
    TERM_THEFT_INTELLECTUAL_PROPERTY = "Theft - Intellectual Property"
    TERM_THEFT_CREDENTIAL_THEFT = "Theft - Credential Theft"
    TERM_THEFT_IDENTITY_THEFT = "Theft - Identity Theft"
    TERM_THEFT_THEFT_OF_PROPRIETARY_INFORMATION = "Theft - Theft of Proprietary Information"
    TERM_ACCOUNT_TAKEOVER = "Account Takeover"
    TERM_BRAND_DAMAGE = "Brand Damage"
    TERM_COMPETITIVE_ADVANTAGE = "Competitive Advantage"
    TERM_DEGRADATION_OF_SERVICE = "Degradation of Service"
    TERM_DENIAL_AND_DECEPTION = "Denial and Deception"
    TERM_DESTRUCTION = "Destruction"
    TERM_DISRUPTION = "Disruption"
    TERM_EMBARRASSMENT = "Embarrassment"
    TERM_EXPOSURE = "Exposure"
    TERM_EXTORTION = "Extortion"
    TERM_FRAUD = "Fraud"
    TERM_HARASSMENT = "Harassment"
    TERM_ICS_CONTROL = "ICS Control"
    TERM_TRAFFIC_DIVERSION = "Traffic Diversion"
    TERM_UNAUTHORIZED_ACCESS = "Unauthorized Access"


@register_vocab
class PackageIntent(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:PackageIntentVocab-1.0'

    TERM_COLLECTIVE_THREAT_INTELLIGENCE = "Collective Threat Intelligence"
    TERM_THREAT_REPORT = "Threat Report"
    TERM_INDICATORS = "Indicators"
    TERM_INDICATORS_PHISHING = "Indicators - Phishing"
    TERM_INDICATORS_WATCHLIST = "Indicators - Watchlist"
    TERM_INDICATORS_MALWARE_ARTIFACTS = "Indicators - Malware Artifacts"
    TERM_INDICATORS_NETWORK_ACTIVITY = "Indicators - Network Activity"
    TERM_INDICATORS_ENDPOINT_CHARACTERISTICS = "Indicators - Endpoint Characteristics"
    TERM_CAMPAIGN_CHARACTERIZATION = "Campaign Characterization"
    TERM_THREAT_ACTOR_CHARACTERIZATION = "Threat Actor Characterization"
    TERM_EXPLOIT_CHARACTERIZATION = "Exploit Characterization"
    TERM_ATTACK_PATTERN_CHARACTERIZATION = "Attack Pattern Characterization"
    TERM_MALWARE_CHARACTERIZATION = "Malware Characterization"
    TERM_TTP_INFRASTRUCTURE = "TTP - Infrastructure"
    TERM_TTP_TOOLS = "TTP - Tools"
    TERM_COURSES_OF_ACTION = "Courses of Action"
    TERM_INCIDENT = "Incident"
    TERM_OBSERVATIONS = "Observations"
    TERM_OBSERVATIONS_EMAIL = "Observations - Email"
    TERM_MALWARE_SAMPLES = "Malware Samples"


@register_vocab
class LossDuration(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:LossDurationVocab-1.0'

    TERM_PERMANENT = "Permanent"
    TERM_WEEKS = "Weeks"
    TERM_DAYS = "Days"
    TERM_HOURS = "Hours"
    TERM_MINUTES = "Minutes"
    TERM_SECONDS = "Seconds"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class OwnershipClass(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:OwnershipClassVocab-1.0'

    TERM_INTERNALLYOWNED = "Internally-Owned"
    TERM_EMPLOYEEOWNED = "Employee-Owned"
    TERM_PARTNEROWNED = "Partner-Owned"
    TERM_CUSTOMEROWNED = "Customer-Owned"
    TERM_UNKNOWN = "Unknown"


@register_vocab
class MalwareType(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:MalwareTypeVocab-1.0'

    TERM_AUTOMATED_TRANSFER_SCRIPTS = "Automated Transfer Scripts"
    TERM_ADWARE = "Adware"
    TERM_DIALER = "Dialer"
    TERM_BOT = "Bot"
    TERM_BOT_CREDENTIAL_THEFT = "Bot - Credential Theft"
    TERM_BOT_DDOS = "Bot - DDoS"
    TERM_BOT_LOADER = "Bot - Loader"
    TERM_BOT_SPAM = "Bot - Spam"
    TERM_DOS_OR_DDOS = "DoS / DDoS"
    TERM_DOS_OR_DDOS_PARTICIPATORY = "DoS / DDoS - Participatory"
    TERM_DOS_OR_DDOS_SCRIPT = "DoS / DDoS - Script"
    TERM_DOS_OR_DDOS_STRESS_TEST_TOOLS = "DoS / DDoS - Stress Test Tools"
    TERM_EXPLOIT_KITS = "Exploit Kits"
    TERM_POS_OR_ATM_MALWARE = "POS / ATM Malware"
    TERM_RANSOMWARE = "Ransomware"
    TERM_REMOTE_ACCESS_TROJAN = "Remote Access Trojan"
    TERM_ROGUE_ANTIVIRUS = "Rogue Antivirus"
    TERM_ROOTKIT = "Rootkit"


@register_vocab
class IncidentEffect(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:IncidentEffectVocab-1.0'

    TERM_BRAND_OR_IMAGE_DEGRADATION = "Brand or Image Degradation"
    TERM_LOSS_OF_COMPETITIVE_ADVANTAGE = "Loss of Competitive Advantage"
    TERM_LOSS_OF_COMPETITIVE_ADVANTAGE_ECONOMIC = "Loss of Competitive Advantage - Economic"
    TERM_LOSS_OF_COMPETITIVE_ADVANTAGE_MILITARY = "Loss of Competitive Advantage - Military"
    TERM_LOSS_OF_COMPETITIVE_ADVANTAGE_POLITICAL = "Loss of Competitive Advantage - Political"
    TERM_DATA_BREACH_OR_COMPROMISE = "Data Breach or Compromise"
    TERM_DEGRADATION_OF_SERVICE = "Degradation of Service"
    TERM_DESTRUCTION = "Destruction"
    TERM_DISRUPTION_OF_SERVICE_OR_OPERATIONS = "Disruption of Service / Operations"
    TERM_FINANCIAL_LOSS = "Financial Loss"
    TERM_LOSS_OF_CONFIDENTIAL_OR_PROPRIETARY_INFORMATION_OR_INTELLECTUAL_PROPERTY = "Loss of Confidential / Proprietary Information or Intellectual Property"
    TERM_REGULATORY_COMPLIANCE_OR_LEGAL_IMPACT = "Regulatory, Compliance or Legal Impact"
    TERM_UNINTENDED_ACCESS = "Unintended Access"
    TERM_USER_DATA_LOSS = "User Data Loss"


@register_vocab
class InformationSourceRole(VocabString):
    _namespace = 'http://stix.mitre.org/default_vocabularies-1'
    _XSI_TYPE = 'stixVocabs:InformationSourceRoleVocab-1.0'

    TERM_INITIAL_AUTHOR = "Initial Author"
    TERM_CONTENT_ENHANCERORREFINER = "Content Enhancer/Refiner"
    TERM_AGGREGATOR = "Aggregator"
    TERM_TRANSFORMERORTRANSLATOR = "Transformer/Translator"




import unittest

from stix.test import EntityTestCase

from stix.extensions.marking.ais import AISMarkingStructure


class AISMarkingStructureNotProprietaryTests(EntityTestCase, unittest.TestCase):
    klass = AISMarkingStructure
    _full_dict = {
        'not_proprietary':
            {
                'cisa_proprietary': 'false',
                'ais_consent': {'consent': 'NONE'},
                'tlp_marking': {'color': 'GREEN'}
            },
        'xsi:type': 'AIS:AISMarkingStructure'
    }


class AISMarkingStructureIsProprietaryTests(EntityTestCase, unittest.TestCase):
    klass = AISMarkingStructure
    _full_dict = {
        'is_proprietary':
            {
                'cisa_proprietary': 'true',
                'ais_consent': {'consent': 'EVERYONE'},
                'tlp_marking': {'color': 'AMBER'}
            },
        'xsi:type': 'AIS:AISMarkingStructure'
    }

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from __future__ import absolute_import

from .structured_text import StructuredText  # noqa
from .vocabs import VocabString   # noqa
from .datetimewithprecision import DateTimeWithPrecision  # noqa
from .activity import Activity  # noqa
from .confidence import Confidence  # noqa
from .identity import Identity  # noqa
from .information_source import InformationSource  # noqa
from .statement import Statement  # noqa
from .tools import ToolInformation  # noqa
from .names import Names  # noqa
from .campaign_reference import CampaignRef  # noqa
from .related import (   # noqa
    GenericRelationshipList, RelatedCampaign, RelatedCOA,
    RelatedExploitTarget, RelatedIdentity, RelatedIncident,
    RelatedIndicator, RelatedObservable, RelatedThreatActor, RelatedTTP,
    RelatedPackage, RelatedPackages, RelatedCampaignRef
)

# Patch in base types of Related* types
from stix.core import STIXPackage
from cybox.core import Observable
from stix.campaign import Campaign
from stix.coa import CourseOfAction
from stix.exploit_target import ExploitTarget
from stix.incident import Incident
from stix.indicator import Indicator
from stix.threat_actor import ThreatActor
from stix.ttp import TTP


RelatedCampaign._base_type = Campaign  # noqa
RelatedCOA._base_type = CourseOfAction  # noqa
RelatedExploitTarget._base_type = ExploitTarget  # noqa
RelatedIdentity._base_type = Identity  # noqa
RelatedIncident._base_type = Incident  # noqa
RelatedIndicator._base_type = Indicator  # noqa
RelatedThreatActor._base_type = ThreatActor  # noqa
RelatedTTP._base_type = TTP  # noqa
RelatedObservable._base_type = Observable  # noqa
RelatedPackage._base_type = STIXPackage  # noqa
RelatedCampaignRef._base_type = CampaignRef  # noqa

# Path the RelatedPackages _contained_type
RelatedPackages._contained_type = RelatedPackage  # noqa


import stix
import stix.utils as utils
import stix.bindings.stix_common as common_binding


class EncodedCDATA(stix.Entity):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = _binding.EncodedCDATAType
    
    def __init__(self, value=None, encoded=None):
        self.value = value
        self.encoded = encoded

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = utils.strip_cdata(value)

    @property
    def cdata(self):
        return utils.cdata(self.value)

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()
        
        return_obj.value = obj.valueOf_
        return_obj.encoded = obj.encoded

        return return_obj
    
    def to_obj(self, return_obj=None, ns_info=None):
        super(EncodedCDATA, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        return_obj.encoded = self.encoded
        return_obj.valueOf_ = utils.cdata(self.value)

        return return_obj
    
    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()
            
        return_obj.value = utils.strip_cdata(d.get('value'))
        return_obj.encoded = bool(d.get('encoded'))

        return return_obj

    def to_dict(self):
        d = {}

        if not self.value:
            return d

        d['value'] = utils.strip_cdata(self.value)
        d['encoded'] = bool(self.encoded)

        return d

    def __str__(self):
        return self.__unicode__().encode("utf-8")

    def __unicode__(self):
        return unicode(self.value)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
from stix.common import Activity, Confidence, Statement, VocabString
from stix.common.related import (
    GenericRelationshipList, RelatedCampaign, RelatedIncident, RelatedIndicator,
    RelatedPackageRefs, RelatedThreatActor, RelatedTTP
)
from stix.common import vocabs
from stix.data_marking import Marking
import stix.bindings.campaign as campaign_binding


class AssociatedCampaigns(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.AssociatedCampaignsType
    _binding_var = "Associated_Campaign"
    _contained_type = RelatedCampaign
    _inner_name = "campaigns"


class Attribution(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.AttributionType
    _binding_var = "Attributed_Threat_Actor"
    _contained_type = RelatedThreatActor
    _inner_name = "threat_actors"


class RelatedIncidents(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.RelatedIncidentsType
    _binding_var = "Related_Incident"
    _contained_type = RelatedIncident
    _inner_name = "incidents"


class RelatedIndicators(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.RelatedIndicatorsType
    _binding_var = "Related_Indicator"
    _contained_type = RelatedIndicator
    _inner_name = "indicators"


class RelatedTTPs(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.RelatedTTPsType
    _binding_var = "Related_TTP"
    _contained_type = RelatedTTP
    _inner_name = "ttps"


class Names(stix.EntityList):
    _namespace = "http://stix.mitre.org/Campaign-1"
    _binding = campaign_binding
    _binding_class = campaign_binding.NamesType
    _binding_var = "Name"
    _contained_type = VocabString
    _inner_name = "names"


class Campaign(stix.BaseCoreComponent):
    _binding = campaign_binding
    _binding_class = _binding.CampaignType
    _namespace = "http://stix.mitre.org/Campaign-1"
    _version = "1.1.1"
    _ALL_VERSIONS = ("1.0", "1.0.1", "1.1", "1.1.1")
    _ID_PREFIX = 'campaign'

    def __init__(self, id_=None, idref=None, timestamp=None, title=None,
                 description=None, short_description=None):

        super(Campaign, self).__init__(
            id_=id_,
            idref=idref,
            timestamp=timestamp,
            title=title,
            description=description,
            short_description=short_description
        )

        self.names = None
        self.intended_effects = _IntendedEffects()
        self.status = None
        self.related_ttps = RelatedTTPs()
        self.related_incidents = RelatedIncidents()
        self.related_indicators = RelatedIndicators()
        self.attribution = _AttributionList()
        self.associated_campaigns = AssociatedCampaigns()
        self.confidence = None
        self.activity = _Activities()
        self.handling = None
        self.related_packages = RelatedPackageRefs()

    @property
    def intended_effects(self):
        return self._intended_effects

    @intended_effects.setter
    def intended_effects(self, value):
        self._intended_effects = _IntendedEffects(value)

    def add_intended_effect(self, value):
        self.intended_effects.append(value)

    @property
    def activity(self):
        return self._activity

    @activity.setter
    def activity(self, value):
        self._activity = _Activities(value)

    def add_activity(self, value):
        self.activity.append(value)

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, value):
        self._set_vocab(vocabs.CampaignStatus, status=value)

    @property
    def attribution(self):
        return self._attribution

    @attribution.setter
    def attribution(self, value):
        self._attribution = _AttributionList(value)

    def to_obj(self, return_obj=None, ns_info=None):
        if not return_obj:
            return_obj = self._binding_class()

        super(Campaign, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if self.names:
            return_obj.Names = self.names.to_obj(ns_info=ns_info)
        if self.intended_effects:
            return_obj.Intended_Effect = self.intended_effects.to_obj(ns_info=ns_info)
        if self.status:
            return_obj.Status = self.status.to_obj(ns_info=ns_info)
        if self.related_ttps:
            return_obj.Related_TTPs = self.related_ttps.to_obj(ns_info=ns_info)
        if self.related_incidents:
            return_obj.Related_Incidents = self.related_incidents.to_obj(ns_info=ns_info)
        if self.related_indicators:
            return_obj.Related_Indicators = self.related_indicators.to_obj(ns_info=ns_info)
        if self.attribution:
            return_obj.Attribution = self.attribution.to_obj(ns_info=ns_info)
        if self.associated_campaigns:
            return_obj.Associated_Campaigns = self.associated_campaigns.to_obj(ns_info=ns_info)
        if self.confidence:
            return_obj.Confidence = self.confidence.to_obj(ns_info=ns_info)
        if self.activity:
            return_obj.Activity = self.activity.to_obj(ns_info=ns_info)
        if self.handling:
            return_obj.Handling = self.handling.to_obj(ns_info=ns_info)
        if self.related_packages:
            return_obj.Related_Packages = self.related_packages.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        super(Campaign, cls).from_obj(obj, return_obj=return_obj)

        if isinstance(obj, cls._binding_class):
            return_obj.names = Names.from_obj(obj.Names)
            return_obj.intended_effects = \
                _IntendedEffects.from_obj(obj.Intended_Effect)
            return_obj.status = VocabString.from_obj(obj.Status)
            return_obj.related_ttps = RelatedTTPs.from_obj(obj.Related_TTPs)
            return_obj.related_incidents = \
                RelatedIncidents.from_obj(obj.Related_Incidents)
            return_obj.related_indicators = \
                RelatedIndicators.from_obj(obj.Related_Indicators)
            return_obj.attribution = _AttributionList.from_obj(obj.Attribution)
            return_obj.associated_campaigns = \
                AssociatedCampaigns.from_obj(obj.Associated_Campaigns)
            return_obj.confidence = Confidence.from_obj(obj.Confidence)
            return_obj.activity = _Activities.from_obj(obj.Activity)
            return_obj.handling = Marking.from_obj(obj.Handling)
            return_obj.related_packages = \
                RelatedPackageRefs.from_obj(obj.Related_Packages)

        return return_obj

    def to_dict(self):
        return super(Campaign, self).to_dict()

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        super(Campaign, cls).from_dict(dict_repr, return_obj=return_obj)

        get = dict_repr.get  # PEP 8 line lengths
        return_obj.names = Names.from_dict(get('names'))
        return_obj.intended_effects = \
            _IntendedEffects.from_dict(get('intended_effects'))
        return_obj.status = VocabString.from_dict(get('status'))
        return_obj.related_ttps = \
            RelatedTTPs.from_dict(get('related_ttps'))
        return_obj.related_incidents = \
            RelatedIncidents.from_dict(get('related_incidents'))
        return_obj.related_indicators = \
            RelatedIndicators.from_dict(get('related_indicators'))
        return_obj.attribution = _AttributionList.from_list(get('attribution'))
        return_obj.associated_campaigns = \
            AssociatedCampaigns.from_dict(get('associated_campaigns'))
        return_obj.confidence = \
            Confidence.from_dict(get('confidence'))
        return_obj.activity = _Activities.from_dict(get('activity'))
        return_obj.handling = Marking.from_dict(get('handling'))
        return_obj.related_packages = \
            RelatedPackageRefs.from_dict(get('related_packages'))

        return return_obj


# Not Actual STIX Types!
class _AttributionList(stix.TypedList):
    _contained_type = Attribution


class _Activities(stix.TypedList):
    _contained_type = Activity


class _IntendedEffects(stix.TypedList):
    _contained_type = Statement

    def _fix_value(self, value):
        intended_effect = vocabs.IntendedEffect(value)
        return Statement(value=intended_effect)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
from stix.common import StructuredText, Confidence
import stix.bindings.course_of_action as coa_binding


class Objective(stix.Entity):
    _binding = coa_binding
    _binding_class = coa_binding.ObjectiveType
    _namespace = "http://stix.mitre.org/CourseOfAction-1"
    
    def __init__(self, description=None, short_description=None):
        self.description = description
        self.short_description = short_description
        self.applicability_confidence = None

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        self._set_var(StructuredText, description=value)

    @property
    def short_description(self):
        return self._short_description

    @short_description.setter
    def short_description(self, value):
        self._set_var(StructuredText, short_description=value)

    @property
    def applicability_confidence(self):
        return self._applicability_confidence
    
    @applicability_confidence.setter
    def applicability_confidence(self, value):
        self._set_var(Confidence, try_cast=False, applicability_confidence=value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(Objective, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        if self.description:
            return_obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.short_description:
            return_obj.Short_Description = self.short_description.to_obj(ns_info=ns_info)
        if self.applicability_confidence:
            return_obj.Applicability_Confidence = self.applicability_confidence.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.description = StructuredText.from_obj(obj.Description)
        return_obj.short_description = StructuredText.from_obj(obj.Short_Description)
        return_obj.applicability_confidence = Confidence.from_obj(obj.Applicability_Confidence)
        return return_obj

    def to_dict(self):
        return super(Objective, self).to_dict()

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if not return_obj:
            return_obj = cls()

        get = dict_repr.get
        return_obj.description = StructuredText.from_dict(get('description'))
        return_obj.short_description = StructuredText.from_dict(get('short_description'))
        return_obj.applicability_confidence = Confidence.from_dict(get('applicability_confidence'))
        
        return return_obj



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# internal
import stix
import stix.bindings.exploit_target as exploit_target_binding
from stix.common.related import (
    GenericRelationshipList, RelatedCOA, RelatedExploitTarget,
    RelatedPackageRefs
)
from stix.data_marking import Marking

# relative
from .vulnerability import Vulnerability, _Vulnerabilities  # noqa
from .weakness import Weakness, _Weaknesses  # noqa
from .configuration import Configuration, _Configurations  # noqa


class ExploitTarget(stix.BaseCoreComponent):
    """Implementation of STIX ``ExploitTarget``.

    Args:
        id_ (optional): An identifier. If ``None``, a value will be generated
            via ``stix.utils.create_id()``. If set, this will unset the
            ``idref`` property.
        idref (optional): An identifier reference. If set this will unset the
            ``id_`` property.
        title (optional): A string title.
        timestamp (optional): A timestamp value. Can be an instance of
            ``datetime.datetime`` or ``str``.
        description (optional): A string description.
        short_description (optional): A string short description.

    """
    _binding = exploit_target_binding
    _binding_class = _binding.ExploitTargetType
    _namespace = "http://stix.mitre.org/ExploitTarget-1"
    _version = "1.1.1"
    _ALL_VERSIONS = ("1.0", "1.0.1", "1.1", "1.1.1")
    _ID_PREFIX = 'et'

    def __init__(self, id_=None, idref=None, timestamp=None, title=None,
                 description=None, short_description=None):

        super(ExploitTarget, self).__init__(
            id_=id_,
            idref=idref,
            timestamp=timestamp,
            title=title,
            description=description,
            short_description=short_description
        )

        self.handling = None
        self.potential_coas = PotentialCOAs()
        self.related_exploit_targets = RelatedExploitTargets()
        self.vulnerabilities = None
        self.weaknesses = None
        self.configuration = None
        self.related_packages = RelatedPackageRefs()

    @property
    def handling(self):
        
        return self._handling

    @handling.setter
    def handling(self, value):
        self._set_var(Marking, try_cast=False, handling=value)
 
    @property
    def vulnerabilities(self):
        """A list of ``Vulnerability`` objects 
        
        Default Value: ``None``

        Returns:
            A list of
            :class:`stix.exploit_target.vulnerability`

        Raises:
            ValueError: If set to a value that is not ``None`` and not an
                instance of
                :class:`stix.exploit_target.vulnerability`

        """
        
        return self._vulnerabilities
    
    @vulnerabilities.setter
    def vulnerabilities(self, value):
        self._vulnerabilities = _Vulnerabilities(value)

    def add_vulnerability(self, v):
        """Adds a vulnerability to the ``vulnerabilies`` list property.

        Note:
            If ``None`` is passed in no value is added 

        Args:
            v: A Vulnerability value.
            
        Raises: ValueError if the ``v`` param is of type
            :class:`stix.exploit_target.vulnerability`
    

        """
        self.vulnerabilities.append(v)
 
    @property
    def weaknesses(self):
        """A list of ``Weakness`` objects 
        
        Default Value: ``None``

        Returns:
            A list of
            :class:`stix.exploit_target.weakness`

        Raises:
            ValueError: If set to a value that is not ``None`` and not an
                instance of
                :class:`stix.exploit_target.weakness`

        """
        
        return self._weaknesses
    
    @weaknesses.setter
    def weaknesses(self, value):
        self._weaknesses = _Weaknesses(value)
            
    def add_weakness(self, v):
        """Adds a weakness to the ``weaknesses`` list property.

        Note:
            If ``None`` is passed in no value is added 

        Args:
            v: A weakness value.
            
        Raises: ValueError if the ``v`` param is of type :class:`stix.exploit_target.weakness`
    

        """
        self.weaknesses.append(v)
 
    @property
    def configuration(self):
        """A list of ``Configuration`` objects 
        
        Default Value: ``None``

        Returns:
            A list of
            :class:`stix.exploit_target.configuration`

        Raises:
            ValueError: If set to a value that is not ``None`` and not an
                instance of
                :class:`stix.exploit_target.configuration`

        """
        
        return self._configuration
    
    @configuration.setter
    def configuration(self, value):
        self._configuration = _Configurations(value)

    def add_configuration(self, v):
        """Adds a configuration to the ``configurations`` list property.

        Note:
            If ``None`` is passed in no value is added 

        Args:
            v: A configuration value.
            
        Raises: ValueError if the ``v`` param is of type :class:`stix.exploit_target.configuration`

        """
        self.configuration.append(v)
 
    def to_obj(self, return_obj=None, ns_info=None):
        if not return_obj:
            return_obj = self._binding_class()

        super(ExploitTarget, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if self.handling:
            return_obj.Handling = self.handling.to_obj(ns_info=ns_info)
        if self.potential_coas:
            return_obj.Potential_COAs = self.potential_coas.to_obj(ns_info=ns_info)
        if self.related_exploit_targets:
            return_obj.Related_Exploit_Targets = self.related_exploit_targets.to_obj(ns_info=ns_info)
        if self.vulnerabilities:
            return_obj.Vulnerability = self.vulnerabilities.to_obj(ns_info=ns_info)
        if self.weaknesses:
            return_obj.Weakness = self.weaknesses.to_obj(ns_info=ns_info)
        if self.configuration:
            return_obj.Configuration = self.configuration.to_obj(ns_info=ns_info)
        if self.related_packages:
            return_obj.Related_Packages = self.related_packages.to_obj(ns_info=ns_info)
            
        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        super(ExploitTarget, cls).from_obj(obj, return_obj=return_obj)

        if isinstance(obj, cls._binding_class):
            return_obj.handling = Marking.from_obj(obj.Handling)
            return_obj.potential_coas = PotentialCOAs.from_obj(obj.Potential_COAs)
            return_obj.related_exploit_targets = RelatedExploitTargets.from_obj(obj.Related_Exploit_Targets)
            return_obj.vulnerabilities = _Vulnerabilities.from_obj(obj.Vulnerability)
            return_obj.weaknesses = _Weaknesses.from_obj(obj.Weakness)
            return_obj.configuration = _Configurations.from_obj(obj.Configuration)
            return_obj.related_packages = RelatedPackageRefs.from_obj(obj.Related_Packages)

        return return_obj

    def to_dict(self):
        return super(ExploitTarget, self).to_dict()

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        super(ExploitTarget, cls).from_dict(dict_repr, return_obj=return_obj)

        get = dict_repr.get
        return_obj.handling = Marking.from_dict(get('handling'))
        return_obj.potential_coas = PotentialCOAs.from_dict(get('potential_coas'))
        return_obj.related_exploit_targets = RelatedExploitTargets.from_dict(get('related_exploit_targets'))
        return_obj.vulnerabilities = _Vulnerabilities.from_dict(get('vulnerabilities'))
        return_obj.weaknesses = _Weaknesses.from_dict(get('weaknesses'))
        return_obj.configuration = _Configurations.from_dict(get('configuration'))
        return_obj.related_packages = RelatedPackageRefs.from_dict(get('related_packages'))
        
        return return_obj


class PotentialCOAs(GenericRelationshipList):
    """
    A list of ``Potential_COA`` objects, defaults to empty array
    """
    _namespace = "http://stix.mitre.org/ExploitTarget-1"
    _binding = exploit_target_binding
    _binding_class = exploit_target_binding.PotentialCOAsType
    _binding_var = "Potential_COA"
    _contained_type = RelatedCOA
    _inner_name = "coas"

    def __init__(self, coas=None, scope=None):
        if coas is None:
            coas = []
        super(PotentialCOAs, self).__init__(scope, *coas)


class RelatedExploitTargets(GenericRelationshipList):
    """
    A list of ``RelatedExploitTargets`` objects, defaults to empty array
    """
    _namespace = "http://stix.mitre.org/ExploitTarget-1"
    _binding = exploit_target_binding
    _binding_class = exploit_target_binding.RelatedExploitTargetsType
    _binding_var = "Related_Exploit_Target"
    _contained_type = RelatedExploitTarget
    _inner_name = "related_exploit_targets"

    def __init__(self, related_exploit_targets=None, scope=None):
        super(RelatedExploitTargets, self).__init__(scope, related_exploit_targets)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# internal
import stix
import stix.bindings.stix_common as common_binding

# relative
from .vocabs import VocabString


class Names(stix.EntityList):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = _binding.NamesType
    _contained_type = VocabString
    _binding_var = 'Name'
    _inner_name = 'names'
    _dict_as_list = True



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from __future__ import absolute_import

import stix
import stix.utils as utils
import stix.bindings.stix_common as common_binding

from .confidence import Confidence
from .structured_text import StructuredText
from .vocabs import VocabString, HighMediumLow


class Statement(stix.Entity):
    _namespace = 'http://stix.mitre.org/common-1'
    _binding = common_binding
    _binding_class = common_binding.StatementType

    def __init__(self, value=None, timestamp=None, description=None,
                 source=None):
        self.timestamp = timestamp or utils.dates.now()
        self.timestamp_precision = "second"
        self.value = value
        self.description = description
        self.source = source
        self.confidence = None

    @property
    def timestamp(self):
        return self._timestamp

    @timestamp.setter
    def timestamp(self, value):
        self._timestamp = utils.dates.parse_value(value)

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._set_vocab(HighMediumLow, value=value)

    @property
    def source(self):
        return self._source

    @source.setter
    def source(self, value):
        from .information_source import InformationSource
        self._set_var(InformationSource, try_cast=False, source=value)
          
    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        self._set_var(StructuredText, description=value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(Statement, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        obj = self._binding_class()

        if self.timestamp:
            obj.timestamp = self.timestamp.isoformat()
        obj.timestamp_precision = self.timestamp_precision
        if self.value:
            obj.Value = self.value.to_obj(ns_info=ns_info)
        if self.description:
            obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.source:
            obj.Source = self.source.to_obj(ns_info=ns_info)
        if self.confidence:
            obj.Confidence = self.confidence.to_obj(ns_info=ns_info)

        return obj

    def to_dict(self):
        d = utils.to_dict(self, skip=('timestamp_precision',))

        if self.timestamp_precision != 'second':
            d['timestamp_precision'] = self.timestamp_precision

        return d

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        from .information_source import InformationSource
        
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.timestamp = obj.timestamp
        return_obj.timestamp_precision = obj.timestamp_precision
        return_obj.value = VocabString.from_obj(obj.Value)
        return_obj.description = StructuredText.from_obj(obj.Description)
        return_obj.source = InformationSource.from_obj(obj.Source)
        return_obj.confidence = Confidence.from_obj(obj.Confidence)

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        from .information_source import InformationSource
        
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.timestamp = d.get('timestamp')
        return_obj.timestamp_precision = d.get('timestamp_precision', 'second')
        return_obj.value = VocabString.from_dict(d.get('value'))
        return_obj.description = StructuredText.from_dict(d.get('description'))
        return_obj.source = InformationSource.from_dict(d.get('source'))
        return_obj.confidence = Confidence.from_dict(d.get('confidence'))

        return return_obj



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.
import stix
import stix.bindings.ttp as ttp_binding

from .malware_instance import MalwareInstance
from .exploit import Exploit
from .attack_pattern import AttackPattern


class Behavior(stix.Entity):
    _binding = ttp_binding
    _binding_class = _binding.BehaviorType
    _namespace = "http://stix.mitre.org/TTP-1"

    def __init__(self, malware_instances=None, attack_patterns=None, exploits=None):
        self.malware_instances = malware_instances
        self.attack_patterns = attack_patterns
        self.exploits = exploits

    @property
    def malware_instances(self):
        return self._malware_instances

    @malware_instances.setter
    def malware_instances(self, value):
        self._malware_instances = MalwareInstances(value)

    def add_malware_instance(self, malware):
        self.malware_instances.append(malware)

    @property
    def attack_patterns(self):
        return self._attack_patterns

    @attack_patterns.setter
    def attack_patterns(self, value):
        self._attack_patterns = AttackPatterns(value)

    def add_attack_pattern(self, attack_pattern):
        self.attack_patterns.append(attack_pattern)

    @property
    def exploits(self):
        return self._exploits

    @exploits.setter
    def exploits(self, value):
        self._exploits = Exploits(value)

    def add_exploit(self, exploit):
        self.exploits.append(exploit)

    def to_obj(self, return_obj=None, ns_info=None):
        super(Behavior, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        if self.malware_instances:
            return_obj.Malware = self.malware_instances.to_obj(ns_info=ns_info)
        if self.exploits:
            return_obj.Exploits = self.exploits.to_obj(ns_info=ns_info)
        if self.attack_patterns:
            return_obj.Attack_Patterns = self.attack_patterns.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.malware_instances = MalwareInstances.from_obj(obj.Malware)
        return_obj.exploits = Exploits.from_obj(obj.Exploits)
        return_obj.attack_patterns = AttackPatterns.from_obj(obj.Attack_Patterns)

        return return_obj

    def to_dict(self):
        return super(Behavior, self).to_dict()

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if not return_obj:
            return_obj = cls()

        get = dict_repr.get

        return_obj.malware_instances = MalwareInstances.from_dict(get('malware_instances'))
        return_obj.exploits = Exploits.from_dict(get('exploits'))
        return_obj.attack_patterns = AttackPatterns.from_dict(get('attack_patterns'))

        return return_obj


class Exploits(stix.EntityList):
    _namespace = "http://stix.mitre.org/TTP-1"
    _contained_type = Exploit
    _binding = ttp_binding
    _binding_class = _binding.ExploitsType
    _binding_var = "Exploit"
    _inner_name = "exploits"
    _dict_as_list = True


class MalwareInstances(stix.EntityList):
    _namespace = "http://stix.mitre.org/TTP-1"
    _contained_type = MalwareInstance
    _binding = ttp_binding
    _binding_class = _binding.MalwareType
    _binding_var = "Malware_Instance"
    _inner_name = "malware_instances"
    _dict_as_list = True


class AttackPatterns(stix.EntityList):
    _namespace = "http://stix.mitre.org/TTP-1"
    _contained_type = AttackPattern
    _binding = ttp_binding
    _binding_class = _binding.AttackPatternsType
    _binding_var = "Attack_Pattern"
    _inner_name = "attack_patterns"
    _dict_as_list = True



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# builtin
import json
import collections
import itertools
import StringIO

# internal
from . import bindings, utils


def _override(*args, **kwargs):
    raise NotImplementedError()


class Entity(object):
    """Base class for all classes in the STIX API."""
    _namespace = None
    _XSI_TYPE = None

    def _collect_ns_info(self, ns_info=None):
        if not ns_info:
            return
        ns_info.collect(self)

    def _set_var(self, klass, try_cast=True, arg=None, **kwargs):
        """Sets an instance property value.

        * If the input value is ``None``, the property is set to ``None``.
        * If the input value is an instance of `klass`, the property is set
          the input value.
        * If the input value is not an instance of `klass` and `try_cast` is
          ``True``, an attempt will be made to cast the input value to an
          instance of `klass`.

        Args:
            klass: The expected input value class.
            try_cast: If ``True`` attempt to cast the input value to `klass`
                if it is not an instance of `klass`.
            arg: The __init__ parameter name to use when casting the input
                value to `klass`. E.g., StructuredText(value=input), the `arg`
                is `value`. If ``None``, it is assumed that the first
                __init__ parameter will accept the value.
            **kwargs: The field name and value. The field name is the key
                and the field value is the value.

        """
        name, item = kwargs.iteritems().next()
        attr = utils.private_name(name)  # 'title' => '_title'

        if item is None:
            setattr(self, attr, None)
        elif isinstance(item, klass):
            setattr(self, attr, item)
        elif try_cast:
            promoted = utils.cast_var(item, klass, arg=arg)
            setattr(self, attr, promoted)
        else:
            error = "'{0}' expects an instance of {1}. Received: {2}."
            error = error.format(name, klass, type(item))
            raise TypeError(error)

    def _set_vocab(self, klass=None, **kwargs):
        """Sets a controlled vocabulary property value.

        * If the input value is ``None``, the property is set to ``None``.
        * If the input value is an instance of ``VocabString``, the property
          is set to the input value.
        * If the input value is not an instance of ``VocabString``, an attempt
          will be made to cast the input to an instance of `klass`. If `klass`
          is ``None``, ``VocabString`` will be used.

        Args:
            klass: The VocabString impl to cast the input value to. If ``None``
                ``VocabString`` will be assumed.
            **kwargs: The field name, value pair. The field name is the key
                and the field value is the value.

        """
        from stix.common import VocabString

        klass = klass or VocabString
        item  = kwargs.itervalues().next()

        if isinstance(item, VocabString):
            self._set_var(VocabString, **kwargs)
        else:
            self._set_var(klass, **kwargs)

    def to_obj(self, return_obj=None, ns_info=None):
        """Converts an `Entity` into a binding object.

        Note:
            This needs to be overridden by derived classes.

        """
        self._collect_ns_info(ns_info)
        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        """Create an object from a binding object"""
        raise NotImplementedError()

    def to_xml(self, include_namespaces=True, include_schemalocs=False,
               ns_dict=None, schemaloc_dict=None, pretty=True,
               auto_namespace=True, encoding='utf-8'):
        """Serializes a :class:`Entity` instance to an XML string.

        The default character encoding is ``utf-8`` and can be set via the
        `encoding` parameter. If `encoding` is ``None``, a unicode string
        is returned.

        Args:
            auto_namespace: Automatically discover and export XML namespaces
                for a STIX :class:`Entity` instance.
            include_namespaces: Export namespace definitions in the output
                XML. Default is ``True``.
            include_schemalocs: Export ``xsi:schemaLocation`` attribute
                in the output document. This will attempt to associate
                namespaces declared in the STIX document with schema locations.
                If a namespace cannot be resolved to a schemaLocation, a
                Python warning will be raised. Schemalocations will only be
                exported if `include_namespaces` is also ``True``.
            ns_dict: Dictionary of XML definitions (namespace is key, alias is
                value) to include in the exported document. This must be
                passed in if `auto_namespace` is ``False``.
            schemaloc_dict: Dictionary of XML ``namespace: schema location``
                mappings to include in the exported document. These will
                only be included if `auto_namespace` is ``False``.
            pretty: Pretty-print the XML.
            encoding: The output character encoding. Default is ``utf-8``. If
                `encoding` is set to ``None``, a unicode string is returned.

        Returns:
            An XML string for this
            :class:`Entity` instance. Default character encoding is ``utf-8``.

        """

        from .utils import nsparser
        parser = nsparser.NamespaceParser()

        if auto_namespace:
            ns_info = nsparser.NamespaceInfo()
        else:
            ns_info = None

        obj = self.to_obj(ns_info=ns_info)

        if (not auto_namespace) and (not ns_dict):
            raise Exception(
                "Auto-namespacing was disabled but ns_dict was empty "
                "or missing."
            )

        if auto_namespace:
            ns_info.finalize(ns_dict=ns_dict, schemaloc_dict=schemaloc_dict)
            obj_ns_dict = ns_info.binding_namespaces
        else:
            ns_info = nsparser.NamespaceInfo()
            ns_info.finalized_namespaces = ns_dict or {}
            ns_info.finalized_schemalocs = schemaloc_dict or {}
            obj_ns_dict = dict(
                itertools.chain(
                    ns_dict.iteritems(),
                    nsparser.DEFAULT_STIX_NAMESPACES.iteritems()
                )
            )

        namespace_def = ""
        if include_namespaces:
            xmlns = parser.get_xmlns_str(ns_info.finalized_namespaces)
            namespace_def += ("\n\t" + xmlns)

        if include_schemalocs and include_namespaces:
            schemaloc = parser.get_schemaloc_str(ns_info.finalized_schemalocs)
            namespace_def += ("\n\t" + schemaloc)

        if not pretty:
            namespace_def = namespace_def.replace('\n\t', ' ')

        with bindings.save_encoding(encoding):
            sio = StringIO.StringIO()
            obj.export(
                sio.write,                    # output buffer
                0,                            # output level
                obj_ns_dict,                  # namespace dictionary
                pretty_print=pretty,          # pretty printing
                namespacedef_=namespace_def   # namespace/schemaloc def string
            )

        # Ensure that the StringIO buffer is unicode
        s = unicode(sio.getvalue())

        if encoding:
            return s.encode(encoding)

        return s

    def to_json(self):
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_doc):
        """Parses the JSON document `json_doc` and returns a STIX
        :class:`Entity` implementation instance.

        Arguments:
            json_doc: Input JSON representation of a STIX entity. This can be
                a readable object or a JSON string.

        Returns:
            An implementation of :class:`.Entity` (e.g.,
            :class:`.STIXPackage`).

        """
        try:
            d = json.load(json_doc)
        except AttributeError:  # catch the read() error
            d = json.loads(json_doc)

        return cls.from_dict(d)

    def to_dict(self):
        """Converts a STIX :class:`Entity` implementation into a Python
        dictionary. This may be overridden by derived classes.

        """
        return utils.to_dict(self)

    @classmethod
    def from_dict(cls, d, return_obj=None):
        """Convert from dict representation to object representation.
        This should be overriden by a subclass

        """
        raise NotImplementedError()
            
    @classmethod
    def object_from_dict(cls, entity_dict):
        """Convert from dict representation to object representation."""
        return cls.from_dict(entity_dict).to_obj()

    @classmethod
    def dict_from_object(cls, entity_obj):
        """Convert from object representation to dict representation."""
        return cls.from_obj(entity_obj).to_dict()

    def walk(self):
        return utils.walk.iterwalk(self)

    def find(self, id_):
        """Searches the children of a :class:`Entity` implementation for an
        object with an ``id_`` property that matches `id_`.

        """
        if not id_:
            return

        for entity in self.walk():
            if id_ == getattr(entity, "id_", None):
                return entity


class EntityList(collections.MutableSequence, Entity):
    _binding_class = _override
    _binding_var = None
    _contained_type = _override
    _inner_name = None
    _dict_as_list = False

    def __init__(self, *args):
        super(EntityList, self).__init__()
        self._inner = []

        if not any(args):
            return

        for arg in args:
            if utils.is_sequence(arg):
                self.extend(arg)
            else:
                self.append(arg)

    def __nonzero__(self):
        return bool(self._inner)

    def __getitem__(self, key):
        return self._inner.__getitem__(key)

    def __setitem__(self, key, value):
        if not self._is_valid(value):
            value = self._fix_value(value)
        self._inner.__setitem__(key, value)

    def __delitem__(self, key):
        self._inner.__delitem__(key)

    def __len__(self):
        return len(self._inner)

    def insert(self, idx, value):
        if not value:
            return
        if not self._is_valid(value):
            value = self._fix_value(value)
        self._inner.insert(idx, value)

    def _is_valid(self, value):
        """Check if this is a valid object to add to the list."""
        # Subclasses can override this function, but if it becomes common, it's
        # probably better to use self._contained_type.istypeof(value)
        return isinstance(value, self._contained_type)

    def _fix_value(self, value):
        """Attempt to coerce value into the correct type.

        Subclasses can override this function.
        """
        try:
            new_value = self._contained_type(value)
        except:
            error = "Can't put '{0}' ({1}) into a {2}. Expected a {3} object."
            error = error.format(
                value,                  # Input value
                type(value),            # Type of input value
                type(self),             # Type of collection
                self._contained_type    # Expected type of input value
            )
            raise ValueError(error)

        return new_value

    # The next four functions can be overridden, but otherwise define the
    # default behavior for EntityList subclasses which define the following
    # class-level members:
    # - _binding_class
    # - _binding_var
    # - _contained_type
    # - _inner_name

    def to_obj(self, return_obj=None, ns_info=None):
        super(EntityList, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        objlist = [x.to_obj(ns_info=ns_info) for x in self]
        setattr(return_obj, self._binding_var, objlist)

        return return_obj

    def to_list(self):
        return [h.to_dict() for h in self]

    def to_dict(self):
        if self._dict_as_list:
            return self.to_list()

        d = utils.to_dict(self, skip=('inner',))

        if self._inner:
            d[self._inner_name] = [h.to_dict() for h in self]

        return d

    @classmethod
    def from_obj(cls, obj, return_obj=None, contained_type=None,
                 binding_var=None):
        if not obj:
            return None

        if return_obj is None:
            return_obj = cls()
        if not contained_type:
            contained_type = cls._contained_type
        if not binding_var:
            binding_var = cls._binding_var

        for item in getattr(obj, binding_var):
            return_obj.append(contained_type.from_obj(item))

        return return_obj

    @classmethod
    def from_list(cls, list_repr, return_obj=None, contained_type=None):

        if not utils.is_sequence(list_repr):
            return None

        if return_obj is None:
            return_obj = cls()
        if not contained_type:
            contained_type = cls._contained_type

        return_obj.extend(contained_type.from_dict(x) for x in list_repr)
        return return_obj

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None, contained_type=None,
                  inner_name=None):

        if cls._dict_as_list:
            return cls.from_list(
                dict_repr,
                return_obj=return_obj,
                contained_type=contained_type,
            )

        if not isinstance(dict_repr, dict):
            return None

        if return_obj is None:
            return_obj = cls()
        if not contained_type:
            contained_type = cls._contained_type
        if not inner_name:
            inner_name = cls._inner_name

        for item in dict_repr.get(inner_name, []):
            return_obj.append(contained_type.from_dict(item))

        return return_obj


class TypedList(collections.MutableSequence):
    _contained_type = _override

    def __init__(self, *args):
        self._inner = []

        # Check if it was initialized with args=None
        if not any(args):
            return

        for arg in args:
            if utils.is_sequence(arg):
                self.extend(arg)
            else:
                self.append(arg)

    def __nonzero__(self):
        return bool(self._inner)

    def __getitem__(self, key):
        return self._inner.__getitem__(key)

    def __setitem__(self, key, value):
        if not self._is_valid(value):
            value = self._fix_value(value)
        self._inner.__setitem__(key, value)

    def __delitem__(self, key):
        self._inner.__delitem__(key)

    def __len__(self):
        return len(self._inner)

    def insert(self, idx, value):
        if not value:
            return
        if not self._is_valid(value):
            value = self._fix_value(value)
        self._inner.insert(idx, value)

    def _is_valid(self, value):
        """Check if this is a valid object to add to the list."""
        # Subclasses can override this function, but if it becomes common,
        # it's probably better to use self._contained_type.istypeof(value)
        return isinstance(value, self._contained_type)

    def _fix_value(self, value):
        """Attempt to coerce value into the correct type.

        Subclasses can override this function.
        """
        try:
            new_value = self._contained_type(value)
        except Exception:
            error = "Can't put '{0}' ({1}) into a {2}. Expected a {3} object."
            error = error.format(
                value,                  # Input value
                type(value),            # Type of input value
                type(self),             # Type of collection
                self._contained_type    # Expected type of input value
            )
            raise ValueError(error)

        return new_value

    def to_obj(self, ns_info=None):
        return [x.to_obj(ns_info=ns_info) for x in self]

    def to_list(self):
        return [h.to_dict() for h in self]

    to_dict = to_list

    @classmethod
    def from_obj(cls, obj_list, contained_type=None):
        if not obj_list:
            return None

        return_obj = cls()

        if not contained_type:
            contained_type = cls._contained_type

        if not utils.is_sequence(obj_list):
            obj_list = [obj_list]

        return_obj.extend(contained_type.from_obj(x) for x in obj_list)
        return return_obj

    @classmethod
    def from_list(cls, list_repr, contained_type=None):

        if not utils.is_sequence(list_repr):
            return None

        return_obj = cls()

        if not contained_type:
            contained_type = cls._contained_type

        return_obj.extend(contained_type.from_dict(x) for x in list_repr)
        return return_obj

    from_dict = from_list

    @classmethod
    def object_from_dict(cls, entity_dict):
        """Convert from dict representation to object representation."""
        return cls.from_dict(entity_dict).to_obj()

    @classmethod
    def dict_from_object(cls, entity_obj):
        """Convert from object representation to dict representation."""
        return cls.from_obj(entity_obj).to_dict()


class BaseCoreComponent(Entity):
    _ALL_VERSIONS = ()
    _ID_PREFIX = None

    def __init__(self, id_=None, idref=None, timestamp=None, title=None,
                 description=None, short_description=None):

        self.id_ = id_ or utils.create_id(self._ID_PREFIX)
        self.idref = idref
        self.title = title
        self.description = description
        self.short_description = short_description
        self.version = None
        self.information_source = None

        if timestamp:
            self.timestamp = timestamp
        else:
            self.timestamp = utils.dates.now() if not idref else None

    @property
    def id_(self):
        """The ``id_`` property serves as an identifier. This is
        automatically set during ``__init__()``.

        Default Value: ``None``

        Note:
            Both the ``id_`` and ``idref`` properties cannot be set at the
            same time. **Setting one will unset the other!**

        Returns:
            A string id.

        """
        return self._id

    @id_.setter
    def id_(self, value):
        if not value:
            self._id = None
        else:
            self._id = value
            self.idref = None

    @property
    def idref(self):
        """The ``idref`` property must be set to the ``id_`` value of another
        object instance of the same type. An idref does not need to resolve to
        a local object instance.

        Default Value: ``None``.

        Note:
            Both the ``id_`` and ``idref`` properties cannot be set at the
            same time. **Setting one will unset the other!**

        Returns:
            The value of the ``idref`` property

        """
        return self._idref

    @idref.setter
    def idref(self, value):
        if not value:
            self._idref = None
        else:
            self._idref = value
            self.id_ = None  # unset id_ if idref is present

    @property
    def version(self):
        """The schematic version of this component. This property
        will always return ``None`` unless it is set to a value different than
        ``self.__class__._version``.

        Note:
            This property refers to the version of the schema component
            type and should not be used for the purpose of content versioning.

        Default Value: ``None``

        Returns:
            The value of the ``version`` property if set to a value different
            than ``self.__class__._version``

        """
        return self._version

    @version.setter
    def version(self, value):
        if not value:
            self._version = None
        else:
            utils.check_version(self._ALL_VERSIONS, value)
            self._version = value

    @property
    def timestamp(self):
        """The timestam property declares the time of creation and is
        automatically set in ``__init__()``.

        This property can accept ``datetime.datetime`` or ``str`` values.
        If an ``str`` value is supplied, a best-effort attempt is made to
        parse it into an instance of ``datetime.datetime``.

        Default Value: A ``datetime.dateime`` instance with a value of the
        date/time when ``__init__()`` was called.

        Note:
            If an ``idref`` is set during ``__init__()``, the value of
            ``timestamp`` will not automatically generated and instead default
            to the ``timestamp`` parameter, which has a default value of
            ``None``.

        Returns:
            An instance of ``datetime.datetime``.

        """
        return self._timestamp

    @timestamp.setter
    def timestamp(self, value):
        self._timestamp = utils.dates.parse_value(value)

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = value

    @property
    def description(self):
        """A description about the contents or purpose of this object.

        Default Value: ``None``

        Note:
            If set to a value that is not an instance of
            :class:`.StructuredText`, an attempt to will be made to convert
            the value into an instance of :class:`.StructuredText`.

        Returns:
            An instance of
            :class:`.StructuredText`

        """
        return self._description

    @description.setter
    def description(self, value):
        from stix.common import StructuredText
        self._set_var(StructuredText, description=value)

    @property
    def short_description(self):
        """A short description about the contents or purpose of this object.

        Default Value: ``None``

        Note:
            If set to a value that is not an instance of
            :class:`.StructuredText`, an attempt to will be made to convert
            the value into an instance of :class:`.StructuredText`.

        Returns:
            An instance of
            :class:`.StructuredText`

        """
        return self._short_description

    @short_description.setter
    def short_description(self, value):
        from stix.common import StructuredText
        self._set_var(StructuredText, short_description=value)

    @property
    def information_source(self):
        """Contains information about the source of this object.

        Default Value: ``None``

        Returns:
            An instance of
            :class:`.InformationSource`

        Raises:
            ValueError: If set to a value that is not ``None`` and not an
                instance of
                :class:`.InformationSource`

        """
        return self._information_source

    @information_source.setter
    def information_source(self, value):
        from stix.common import InformationSource
        self._set_var(InformationSource, try_cast=False, information_source=value)

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        from stix.common import StructuredText, InformationSource

        if not return_obj:
            raise ValueError("Must provide a return_obj argument")

        if not obj:
            raise ValueError("Must provide an obj argument")

        return_obj.id_ = obj.id
        return_obj.idref = obj.idref
        return_obj.timestamp = obj.timestamp

        # These may not be found on the input obj if it isn't a full
        # type definition (e.g., used as a reference)
        return_obj.version = getattr(obj, 'version', None)
        return_obj.title = getattr(obj, 'Title', None)
        return_obj.description = \
            StructuredText.from_obj(getattr(obj, 'Description', None))
        return_obj.short_description = \
            StructuredText.from_obj(getattr(obj, 'Short_Description', None))
        return_obj.information_source = \
            InformationSource.from_obj(getattr(obj, 'Information_Source', None))

        return return_obj

    def to_obj(self, return_obj=None, ns_info=None):
        if not return_obj:
            raise ValueError("Must provide a return_obj argument")

        super(BaseCoreComponent, self).to_obj(
            return_obj=return_obj,
            ns_info=ns_info
        )

        return_obj.id = self.id_
        return_obj.idref = self.idref
        return_obj.version = self.version
        return_obj.Title = self.title

        if self.timestamp:
            return_obj.timestamp = utils.dates.serialize_value(self.timestamp)
        if self.description:
            return_obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.short_description:
            return_obj.Short_Description = self.short_description.to_obj(ns_info=ns_info)
        if self.information_source:
            return_obj.Information_Source = self.information_source.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        from stix.common import StructuredText, InformationSource

        if not return_obj:
            raise ValueError("Must provide a return_obj argument")

        get = d.get
        return_obj.id_ = get('id')
        return_obj.idref = get('idref')
        return_obj.timestamp = get('timestamp')
        return_obj.version = get('version')
        return_obj.title = get('title')
        return_obj.description = \
            StructuredText.from_dict(get('description'))
        return_obj.short_description = \
            StructuredText.from_dict(get('short_description'))
        return_obj.information_source = \
            InformationSource.from_dict(get('information_source'))

        return return_obj

    def to_dict(self):
        return super(BaseCoreComponent, self).to_dict()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# internal
import stix
from stix.common import ToolInformation, Identity
import stix.bindings.ttp as ttp_binding

# relative
from .infrastructure import Infrastructure


class Resource(stix.Entity):
    _binding = ttp_binding
    _binding_class = _binding.ResourceType
    _namespace = "http://stix.mitre.org/TTP-1"

    def __init__(self, tools=None, infrastructure=None, personas=None):
        self.tools = tools
        self.infrastructure = infrastructure
        self.personas = personas

    @property
    def tools(self):
        return self._tools

    @tools.setter
    def tools(self, value):
        self._tools = Tools(value)

    def add_tool(self, tool):
        self.tools.append(tool)

    @property
    def infrastructure(self):
        return self._infrastructure

    @infrastructure.setter
    def infrastructure(self, value):
        self._infrastructure = value

    @property
    def personas(self):
        return self._personas

    @personas.setter
    def personas(self, value):
        self._personas = Personas(value)

    def add_persona(self, persona):
        self.personas.append(persona)

    def to_obj(self, return_obj=None, ns_info=None):
        super(Resource, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        if self.tools:
            return_obj.Tools = self.tools.to_obj(ns_info=ns_info)
        if self.infrastructure:
            return_obj.Infrastructure = self.infrastructure.to_obj(ns_info=ns_info)
        if self.personas:
            return_obj.Personas = self.personas.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.infrastructure = Infrastructure.from_obj(obj.Infrastructure)
        return_obj.tools = Tools.from_obj(obj.Tools)
        return_obj.personas = Personas.from_obj(obj.Personas)

        return return_obj

    def to_dict(self):
        return super(Resource, self).to_dict()

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        get = dict_repr.get

        return_obj.tools = Tools.from_dict(get('tools'))
        return_obj.infrastructure = Infrastructure.from_dict(get('infrastructure'))
        return_obj.personas = Personas.from_dict(get('personas'))

        return return_obj


class Personas(stix.EntityList):
    _namespace = "http://stix.mitre.org/TTP-1"
    _contained_type = Identity
    _binding = ttp_binding
    _binding_class = _binding.PersonasType
    _binding_var = "Persona"
    _inner_name = "personas"
    _dict_as_list = True

    def _fix_value(self, value):
        return Identity(name=value)


class Tools(stix.EntityList):
    _namespace = "http://stix.mitre.org/TTP-1"
    _contained_type = ToolInformation
    _binding = ttp_binding
    _binding_class = _binding.ToolsType
    _binding_var = "Tool"
    _inner_name = "tools"
    _dict_as_list = True



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from __future__ import absolute_import

import stix
import stix.utils as utils
import stix.bindings.stix_common as common_binding

from . import vocabs, VocabString
from .structured_text import StructuredText


class Confidence(stix.Entity):
    _namespace = 'http://stix.mitre.org/common-1'
    _binding = common_binding
    _binding_class = common_binding.ConfidenceType

    def __init__(self, value=None, timestamp=None, description=None, source=None):
        self.timestamp = timestamp or utils.dates.now()
        self.timestamp_precision = "second"
        self.value = value
        self.description = description
        self.source = source
        # TODO: support confidence_assertion_chain
        # self.confidence_assertion_chain = None

    @property
    def timestamp(self):
        return self._timestamp

    @timestamp.setter
    def timestamp(self, value):
        self._timestamp = utils.dates.parse_value(value)

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._set_vocab(vocabs.HighMediumLow, value=value)

    @property
    def source(self):
        return self._source

    @source.setter
    def source(self, value):
        from .information_source import InformationSource
        self._set_var(InformationSource, try_cast=False, source=value)

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        self._set_var(StructuredText, description=value)

    # @property
    # def confidence_assertion_chain(self):
    #     return self._confidence_assertion_chain

    # @confidence_assertion_chain.setter
    # def confidence_assertion_chain(self, value):
    #     if value:
    #         raise NotImplementedError()

    def to_obj(self, return_obj=None, ns_info=None):
        super(Confidence, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        obj = self._binding_class()
        obj.timestamp = utils.dates.serialize_value(self.timestamp)
        obj.timestamp_precision = self.timestamp_precision

        if self.value:
            obj.Value = self.value.to_obj(ns_info=ns_info)
        if self.description:
            obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.source:
            obj.Source = self.source.to_obj(ns_info=ns_info)

        return obj

    def to_dict(self):
        d = utils.to_dict(self, skip=('timestamp_precision',))

        if self.timestamp_precision != 'second':
            d['timestamp_precision'] = self.timestamp_precision

        return d

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        from .information_source import InformationSource
        
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.timestamp = obj.timestamp
        return_obj.timestamp_precision = obj.timestamp_precision
        return_obj.value = VocabString.from_obj(obj.Value)
        return_obj.description = StructuredText.from_obj(obj.Description)
        return_obj.source = InformationSource.from_obj(obj.Source)

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        from .information_source import InformationSource
        
        if not d:
            return None
        
        if not return_obj:
            return_obj = cls()

        return_obj.timestamp = d.get('timestamp')
        return_obj.timestamp_precision = d.get('timestamp_precision', 'second')
        return_obj.value = VocabString.from_dict(d.get('value'))
        return_obj.description = StructuredText.from_dict(d.get('description'))
        return_obj.source = InformationSource.from_dict(d.get('source'))

        return return_obj


# class ConfidenceAssertionChain(stix.Entity):
#     _namespace = 'http://stix.mitre.org/common-2'
#     _binding = common_binding
#
#     def __init__(self):
#         self.confidence_assertions = []
#
#     def add_confidence_assertion(self, confidence_assertion):
#         if isinstance(confidence_assertion, Confidence):
#             self.confidence_assertions.append(confidence_assertion)
#         else:
#             tmp_confidence = Confidence(value=confidence_assertion)
#             self.confidence_assertions.append(tmp_confidence)
#
#     def to_obj(self, return_obj=None, ns_info=None):
#         super(ConfidenceAssertionChain, self).to_obj(return_obj=return_obj, ns_info=ns_info)
#
#         if not return_obj:
#             return_obj = self._binding.ConfidenceAssertionChainType()
#
#         return None
#
#     @classmethod
#     def from_obj(cls, obj, return_obj=None):
#         return None
#
#     def to_dict(self):
#         return {}
#
#     @classmethod
#     def from_dict(cls, dict_repr, return_obj=None):
#         return None



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# internal
import stix
import stix.bindings.ttp as ttp_binding
from stix.common import vocabs, Statement
from stix.data_marking import Marking

# relative
from .behavior import Behavior
from .resource import Resource
from .victim_targeting import VictimTargeting


class TTP(stix.BaseCoreComponent):
    _binding = ttp_binding
    _binding_class = _binding.TTPType
    _namespace = "http://stix.mitre.org/TTP-1"
    _version = "1.1.1"
    _ALL_VERSIONS = ("1.0", "1.0.1", "1.1", "1.1.1")
    _ID_PREFIX = "ttp"

    def __init__(self, id_=None, idref=None, timestamp=None, title=None,
                 description=None, short_description=None):

        super(TTP, self).__init__(
            id_=id_,
            idref=idref,
            timestamp=timestamp,
            title=title,
            description=description,
            short_description=short_description
        )

        self.behavior = None
        self.related_ttps = None
        self.intended_effects = None
        self.resources = None
        self.victim_targeting = None
        self.handling = None
        self.exploit_targets = ExploitTargets()

    @property
    def behavior(self):
        return self._behavior

    @behavior.setter
    def behavior(self, value):
        self._set_var(Behavior, try_cast=False, behavior=value)

    @property
    def related_ttps(self):
        return self._related_ttps

    @related_ttps.setter
    def related_ttps(self, value):
        if isinstance(value, RelatedTTPs):
            self._related_ttps = value
        else:
            self._related_ttps = RelatedTTPs(value)

    @property
    def exploit_targets(self):
        return self._exploit_targets

    @exploit_targets.setter
    def exploit_targets(self, value):
        if isinstance(value, ExploitTargets):
            self._exploit_targets = value
        else:
            self._exploit_targets = ExploitTargets(value)

    @property
    def intended_effects(self):
        return self._intended_effects

    @intended_effects.setter
    def intended_effects(self, value):
        self._intended_effects = _IntendedEffects(value)

    def add_intended_effect(self, value):
        self.intended_effects.append(value)

    @property
    def resources(self):
        return self._resources

    @resources.setter
    def resources(self, value):
        self._set_var(Resource, resources=value)

    @property
    def victim_targeting(self):
        return self._victim_targeting

    @victim_targeting.setter
    def victim_targeting(self, value):
        self._set_var(VictimTargeting, try_cast=False, victim_targeting=value)

    @property
    def handling(self):
        return self._handling

    @handling.setter
    def handling(self, value):
        self._set_var(Marking, try_cast=False, handling=value)

    def to_obj(self, return_obj=None, ns_info=None):
        if not return_obj:
            return_obj = self._binding_class()

        super(TTP, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if self.behavior:
            return_obj.Behavior = self.behavior.to_obj(ns_info=ns_info)
        if self.related_ttps:
            return_obj.Related_TTPs = self.related_ttps.to_obj(ns_info=ns_info)
        if self.exploit_targets:
            return_obj.Exploit_Targets = self.exploit_targets.to_obj(ns_info=ns_info)
        if self.intended_effects:
            return_obj.Intended_Effect = self.intended_effects.to_obj(ns_info=ns_info)
        if self.resources:
            return_obj.Resources = self.resources.to_obj(ns_info=ns_info)
        if self.victim_targeting:
            return_obj.Victim_Targeting = self.victim_targeting.to_obj(ns_info=ns_info)
        if self.handling:
            return_obj.Handling = self.handling.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        super(TTP, cls).from_obj(obj, return_obj=return_obj)

        if isinstance(obj, cls._binding_class):
            return_obj.behavior = Behavior.from_obj(obj.Behavior)
            return_obj.related_ttps = RelatedTTPs.from_obj(obj.Related_TTPs)
            return_obj.exploit_targets = ExploitTargets.from_obj(obj.Exploit_Targets)
            return_obj.resources = Resource.from_obj(obj.Resources)
            return_obj.victim_targeting = VictimTargeting.from_obj(obj.Victim_Targeting)
            return_obj.handling = Marking.from_obj(obj.Handling)
            return_obj.intended_effects = _IntendedEffects.from_obj(obj.Intended_Effect)

        return return_obj

    def to_dict(self):
        return super(TTP, self).to_dict()

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        super(TTP, cls).from_dict(dict_repr, return_obj=return_obj)

        get = dict_repr.get
        return_obj.behavior = Behavior.from_dict(get('behavior'))
        return_obj.related_ttps = RelatedTTPs.from_dict(get('related_ttps'))
        return_obj.exploit_targets = ExploitTargets.from_dict(get('exploit_targets'))
        return_obj.intended_effects = _IntendedEffects.from_dict(get('intended_effects'))
        return_obj.resources = Resource.from_dict(get('resources'))
        return_obj.victim_targeting = VictimTargeting.from_dict(get('victim_targeting'))
        return_obj.handling = Marking.from_dict(get('handling'))

        return return_obj


# NOT ACTUAL STIX TYPE
class _IntendedEffects(stix.TypedList):
    _contained_type = Statement

    def _fix_value(self, value):
        intended_effect = vocabs.IntendedEffect(value)
        return Statement(value=intended_effect)


# Avoid circular imports
from .related_ttps import RelatedTTPs
from .exploit_targets import ExploitTargets



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
from stix.common import InformationSource
import stix.bindings.data_marking as stix_data_marking_binding


class Marking(stix.Entity):
    _binding = stix_data_marking_binding
    _binding_class = stix_data_marking_binding.MarkingType
    _namespace = 'http://data-marking.mitre.org/Marking-1'

    def __init__(self, markings=None):
        self.markings = _MarkingSpecifications(markings)

    @property
    def markings(self):
        return self._markings

    @markings.setter
    def markings(self, value):
        self._markings = _MarkingSpecifications(value)

    def add_marking(self, value):
        self._markings.append(value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(Marking, self).to_obj(
            return_obj=return_obj,
            ns_info=ns_info
        )

        obj = self._binding_class()

        if self.markings:
            obj.Marking = self.markings.to_obj(ns_info=ns_info)

        return obj

    def to_list(self):
        return self.markings.to_list() if self.markings else []

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.markings = _MarkingSpecifications.from_obj(obj.Marking)

        return return_obj

    @classmethod
    def from_list(cls, markings_list, return_obj=None):
        if not markings_list:
            return None

        if not return_obj:
            return_obj = cls()

        mlist = _MarkingSpecifications.from_list(markings_list)
        return_obj.markings = mlist

        return return_obj

    to_dict = to_list
    from_dict = from_list


class MarkingSpecification(stix.Entity):
    _binding = stix_data_marking_binding
    _binding_class = stix_data_marking_binding.MarkingSpecificationType
    _namespace = 'http://data-marking.mitre.org/Marking-1'

    def __init__(self, controlled_structure=None, marking_structures=None):
        self.id_ = None
        self.idref = None
        self.version = None
        self.controlled_structure = controlled_structure
        self.marking_structures = _MarkingStructures(marking_structures)
        self.information_source = None

    @property
    def information_source(self):
        return self._information_source

    @information_source.setter
    def information_source(self, value):
        self._set_var(InformationSource, try_cast=False, information_source=value)

    @property
    def marking_structures(self):
        return self._marking_structures

    @marking_structures.setter
    def marking_structures(self, value):
        self._marking_structures = _MarkingStructures(value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(MarkingSpecification, self).to_obj(
            return_obj=return_obj,
            ns_info=ns_info
        )

        obj = self._binding_class()

        obj.id = self.id_
        obj.idref = self.idref
        obj.version = self.version
        obj.Controlled_Structure = self.controlled_structure
        obj.Marking_Structure = self.marking_structures.to_obj(ns_info=ns_info)

        if self.information_source:
            obj.Information_Source = self.information_source.to_obj(ns_info=ns_info)

        return obj

    def to_dict(self):
        return super(MarkingSpecification, self).to_dict()

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.id_ = obj.id
        return_obj.idref = obj.idref
        return_obj.version = obj.version
        return_obj.controlled_structure = obj.Controlled_Structure
        return_obj.marking_structures = _MarkingStructures.from_obj(obj.Marking_Structure)
        return_obj.information_source = InformationSource.from_obj(obj.Information_Source)

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()

        get = d.get  # PEP8 line length fix
        return_obj.id_ = get('id')
        return_obj.idref = get('idref')
        return_obj.version = get('version')
        return_obj.controlled_structure = get('controlled_structure')
        return_obj.marking_structures = _MarkingStructures.from_dict(
            get('marking_structures')
        )
        return_obj.information_source = InformationSource.from_dict(
            get('information_source')
        )

        return return_obj


class MarkingStructure(stix.Entity):
    _binding = stix_data_marking_binding
    _binding_class = stix_data_marking_binding.MarkingStructureType
    _namespace = 'http://data-marking.mitre.org/Marking-1'
    _XSI_TYPE = None  # overridden by subclasses

    def __init__(self):
        self.id_ = None
        self.idref = None
        self.marking_model_name = None
        self.marking_model_ref = None

    def to_obj(self, return_obj=None, ns_info=None):
        super(MarkingStructure, self).to_obj(
            return_obj=return_obj,
            ns_info=ns_info
        )

        if not return_obj:
            return_obj = self._binding_class()

        return_obj.id = self.id_
        return_obj.idref = self.idref
        return_obj.marking_model_name = self.marking_model_name
        return_obj.marking_model_ref = self.marking_model_ref

        return return_obj

    def to_dict(self):
        d = {}

        if self._XSI_TYPE:
            d['xsi:type'] = self._XSI_TYPE

        if self.id_:
            d['id'] = self.id_
        if self.idref:
            d['idref'] = self.idref
        if self.marking_model_name:
            d['marking_model_name'] = self.marking_model_name
        if self.marking_model_ref:
            d['marking_model_ref'] = self.marking_model_ref

        return d

    @staticmethod
    def lookup_class(xsi_type):
        if not xsi_type:
            return MarkingStructure

        for (k, v) in _EXTENSION_MAP.iteritems():
            # TODO: for now we ignore the prefix and just check for
            # a partial match
            if xsi_type in k:
                return v

        raise ValueError("Unregistered xsi:type %s" % xsi_type)

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        import stix.extensions.marking.tlp  # noqa
        import stix.extensions.marking.simple_marking  # noqa
        import stix.extensions.marking.terms_of_use_marking  # noqa

        if not obj:
            return None

        if return_obj:
            m = return_obj
            m.id_ = obj.id
            m.idref = obj.idref
            m.marking_model_name = obj.marking_model_name
            m.marking_model_ref = obj.marking_model_ref

        else:
            if hasattr(obj, 'xml_type'):
                klass = MarkingStructure.lookup_class(obj.xml_type)
                m = klass.from_obj(obj)
            else:
                m = cls.from_obj(obj, cls())

        return m

    @classmethod
    def from_dict(cls, d, return_obj=None):
        import stix.extensions.marking.tlp  # noqa
        import stix.extensions.marking.simple_marking  # noqa
        import stix.extensions.marking.terms_of_use_marking  # noqa
        
        if not d:
            return None

        if return_obj is not None:
            m = return_obj
            m.id_ = d.get('id')
            m.idref = d.get('idref')
            m.marking_model_name = d.get('marking_model_name')
            m.marking_model_ref = d.get('marking_model_ref')
        else:
            if 'xsi:type' in d:
                cls = MarkingStructure.lookup_class(d.get('xsi:type'))
                m = cls.from_dict(d)
            else:
                m = cls.from_dict(d, cls())

        return m


# Not Actual STIX Types!
class _MarkingSpecifications(stix.TypedList):
    _contained_type = MarkingSpecification


class _MarkingStructures(stix.TypedList):
    _contained_type = MarkingStructure


#: Mapping of marking extension types to classes
_EXTENSION_MAP = {}


def add_extension(cls):
    _EXTENSION_MAP[cls._XSI_TYPE] = cls  # noqa
